      *=====================================================================*
      *  WSDL2RPG: UnMarshaller/Marshaller                                  *
      *=====================================================================*
      *  Author  :  Thomas Raddatz                                          *
      *  Date    :  06.11.2007                                              *
      *  E-mail  :  thomas.raddatz@Tools400.de                              *
      *  Homepage:  www.tools400.de                                         *
      *=====================================================================*
      *                                                                     *
      *  This software is free software, you can redistribute it and/or     *
      *  modify it under the terms of the GNU General Public License (GPL)  *
      *  as published by the Free Software Foundation.                      *
      *                                                                     *
      *  See GNU General Public License for details.                        *
      *          http://www.opensource.org                                  *
      *          http://www.opensource.org/licenses/gpl-license.html        *
      *                                                                     *
      *=====================================================================*
      *  History:                                                           *
      *                                                                     *
      *  Date        Name          Description                              *
      *  ----------  ------------  ---------------------------------------  *
      *  19.03.2008  Th.Raddatz    Added Marshaller/UnMarshaller for        *
      *                            'float' data type.                       *
      *                                                                     *
      *  14.05.2008  Th.Raddatz    Now asking the Controller for the        *
      *                            default RPG string length.               *
      *                            See: Controller_getDefaultStringLength() *
      *                                                                     *
      *  03.09.2008  Th.Raddatz    Added Marshallers/UnMarshallers for      *
      *                            'date', 'time',                          *
      *                            'long', 'unsigned short', 'unsigned int' *
      *                            'unsigned long', 'nonNegativeInteger',   *
      *                            'nonPositiveInteger', 'negativeInteger', *
      *                            'positiveInteger' data types.            *
      *                                                                     *
      *  08.09.2008  Th.Raddatz    Now using AppMsg_sendEscMsg() to signal  *
      *                            errors.                                  *
      *                                                                     *
      *  16.05.2009  Th.Raddatz    Fixed problem in UnMarshaller_toDateTime.*
      *                            Now setting inTimestamp.dot to '.'.      *
      *                                                                     *
      *  06.06.2009  Th.Raddatz    Enhanced the generator to pick up the    *
      *                            content of <schema> and <any> elements.  *
      *                            Added procedures:                        *
      *                               Schema_initialize                     *
      *                               Schema_start                          *
      *                               Schema_end                            *
      *                               Any_initialize                        *
      *                               Any_start                             *
      *                               Any_end                               *
      *                            Added some helper procedures.            *
      *                                                                     *
      *  04.08.2009  Th.Raddatz    Added Marshallers/UnMarshallers for      *
      *                            'integer' data type.                     *
      *                                                                     *
      *  06.12.2009  Th.Raddatz    Fixed MCH1210 (receiver value too small  *
      *                            to hold result) error in procedure       *
      *                            UnMarshaller_toBase64Binary().           *
      *                            'outLength' was defined by mistake as    *
      *                            integer instead of unsigned integer.     *
      *                                                                     *
      *  09.04.2010  Th.Raddatz    Fixed RNX0112 (Date, Time or Timestamp   *
      *                            value is not valid.) in procedure        *
      *                            UnMarshaller_toDate().                   *
      *                                                                     *
      *  11.04.2010  Th.Raddatz    Changed UnMarshaller_toDate() to adjust  *
      *                            the incoming date to the local time zone.*
      *                            Changed UnMarshaller_toTime() to adjust  *
      *                            the incoming date to the local time zone.*
      *                            Changed UnMarshaller_toDateTime() to     *
      *                            adjust the incoming date to the local    *
      *                            time zone.                               *
      *                                                                     *
      *  12.04.2010  Th.Raddatz    Added parameter i_timeZone to:           *
      *                               Marshaller_toDateTime()               *
      *                               Marshaller_toDate()                   *
      *                               Marshaller_toTime()                   *
      *                            Use WSDL_LOCAL_TIMEZONE to add the       *
      *                            local time zone offset.                  *
      *                                                                     *
      *  16.05.2009  Th.Raddatz    Fixed problem in UnMarshaller_toDateTime *
      *                            Now also parsing microseconds from XML   *
      *                            value.                                   *
      *                                                                     *
      *  06.06.2010  Th.Raddatz    Added Marshallers/UnMarshallers for      *
      *                            'gDay', 'gMonth', 'gMonthDay',           *
      *                            'gYear' and 'gYearMonth'.                *
      *                                                                     *
      *  20.08.2010  Th.Raddatz    Added Marshallers/UnMarshallers for      *
      *                            'byte', 'unsigned byte'.                 *
      *                                                                     *
      *  26.10.2010  Th.Raddatz    Added procedure:                         *
      *                            Marshaller_toBase64BinaryStream()        *
      *                                                                     *
      *  23.11.2010  Th.Raddatz    Added Marshallers/UnMarshallers for      *
      *                            'AttributedURI'.                         *
      *                                                                     *
      *  08.12.2010  Th.Raddatz    Added Marshallers/UnMarshallers for      *
      *                            'normalizedString'.                      *
      *                                                                     *
      *  23.02.2011  Th.Raddatz    Changed Marshaller_toDateTime() to       *
      *                            add the local time zone if i_timeZone    *
      *                            has not been passed. Pior to that change *
      *                            it added "Z" to indicate UTC time.       *
      *                                                                     *
      *  23.02.2011  Th.Raddatz    Changed the following procedures to      *
      *                            add the local time zone if i_timeZone    *
      *                            has not been passed:                     *
      *                               Marshaller_toDate                     *
      *                               Marshaller_toTime                     *
      *                               Marshaller_togDay                     *
      *                               Marshaller_togMonth                   *
      *                               Marshaller_togMonthDay                *
      *                               Marshaller_togYear                    *
      *                               Marshaller_togYearMonth               *
      *                                                                     *
      *  23.02.2011  Th.Raddatz    Fixed problem with %subdt() in:          *
      *                               adjustGYear()                         *
      *                               adjustGMonth()                        *
      *                                                                     *
      *  23.02.2011  Th.Raddatz    Changed UnMarshaller_toTime() to use     *
      *                            parseXmlTime() to parse the XML time     *
      *                            value.                                   *
      *                                                                     *
      *  08.06.2011  Th.Raddatz    Added procedure:                         *
      *                            Marshaller_toBase64BinaryBuffer()        *
      *                                                                     *
      *  08.07.2011  Th.Raddatz    Changed procedure Marshaller_toDecimal() *
      *                            to properly remove trailing '0' from     *
      *                            the formatted string.                    *
      *                                                                     *
      *  10.07.2011  Th.Raddatz    Added procedure:                         *
      *                            UnMarshaller_toBase64BinaryStream()      *
      *                                                                     *
      *  14.07.2011  Th.Raddatz    Changed XmlEntityEncoder_encode() to     *
      *                            support large strings of V6R1.           *
      *                                                                     *
      *  17.07.2011  Th.Raddatz    Added marshallers for V6R1 large strings.*
      *                               Marshaller_toAnyTypeV6                *
      *                               Marshaller_toAnyV6                    *
      *                               Marshaller_toStringV6                 *
      *                               Marshaller_toNormalizedStringV6       *
      *                               Marshaller_toLanguageV6               *
      *                               Marshaller_toTokenV6                  *
      *                               Marshaller_toAnyURIV6                 *
      *                               Marshaller_toAttributedURIV6          *
      *                                                                     *
      *  19.07.2011  Th.Raddatz    Added V6 procedures to capture the data  *
      *                            of 'schema' and 'any' types.             *
      *                                                                     *
      *  22.07.2011  Th.Raddatz    Added Marshaller_toSchema() and fixed    *
      *                            problem that Schema_start() and          *
      *                            Schema_end() incorrectly checked 'depth' *
      *                            and 'name'. Removed check for 'name'.    *
      *                                                                     *
      *  30.11.2011  Th.Raddatz    Added marshaller and unmarshaller for    *
      *                            'AnySimpleType'.                         *
      *                                                                     *
      *  98.01.2012  Th.Raddatz    Replaced AppMsg_sendEscMsg() with        *
      *                            AppMsg_sendEscMsgAndDump().              *
      *                                                                     *
      *  02.02.2012  Th.Raddatz    Enhanced the debug information that is   *
      *                            appended to the HTTPAPI debug log.       *
      *                                                                     *
      *  19.06.2012  Th.Raddatz    Added marshaller and unmarshaller for    *
      *                            'QName'.                                 *
      *                                                                     *
      *  09.10.2012  Th.Raddatz    Removed Marshaller_toSchema() and        *
      *                            Marshaller_toSchemaV6().                 *
      *                                                                     *
      *  08.04.2013  Th.Raddatz    Renamed:                                 *
      *                              UnMarshaller_toBase64BinaryStream() to *
      *                              UnMarshaller_fromBase64BufferToStmf()  *
      *                            Renamed:                                 *
      *                              Marshaller_toBase64BinaryBuffer() to   *
      *                              Marshaller_fromBufferToBase64Descriptor*
      *                            Renamed:                                 *
      *                              Marshaller_toBase64BinaryStream() to   *
      *                              Marshaller_fromFileToBase64Descriptor()*
      *                            Added:                                   *
      *                              Marshaller_toBase64BinaryV6() and      *
      *                              UnMarshaller_toBase64BinaryV6()        *
      *                                                                     *
      *  13.04.2013  Th.Raddatz    Added missing clearFieldIdentifier()     *
      *                            in 'on-error' sections.                  *
      *                            Added procedure getFieldNameV6() due to  *
      *                            changed message description USR5020.     *
      *                                                                     *
      *  01.07.2013  Th.Raddatz    Renamed XmlEntityEncoder_encode() and    *
      *                            XmlEntityEncoder_encodeToStream() to     *
      *                            encodeXmlEntityToFieldV6() and           *
      *                            encodeXmlEntityToStream().               *
      *                                                                     *
      *  29.11.2014  Th.Raddatz    Added option WSDL_NO_TIMEZONE.           *
      *                                                                     *
      *=====================================================================*
      *   >>PRE-COMPILER<<                                                  *
      *                                                                     *
      *     >>CRTCMD<<  CRTRPGMOD   MODULE(&LI/&OB) +                       *
      *                             SRCFILE(&SL/&SF) +                      *
      *                             SRCMBR(&SM);                            *
      *                                                                     *
      *     >>COMPILE<<                                                     *
      *       >>PARM<< TRUNCNBR(*NO);                                       *
      *       >>PARM<< DBGVIEW(*LIST);                                      *
      *       >>PARM<< TGTRLS(V5R4M0  );                                    *
      *     >>END-COMPILE<<                                                 *
      *                                                                     *
      *     >>EXECUTE<<                                                     *
      *                                                                     *
      *   >>END-PRE-COMPILER<<                                              *
      *=====================================================================*
 CPY H NOMAIN
      /DEFINE NO_DECEDIT
      /COPY QWSDL2RPG,H_SPEC
     H DECEDIT('0.')
      /UNDEFINE NO_DECEDIT
      /COPY QWSDL2RPG,COPYRIGHT
      /DEFINE COPYRIGHT_DSPEC
      /COPY QWSDL2RPG,COPYRIGHT
      *=====================================================================*
      *
      * ------------------------------------
      *  Constants
      * ------------------------------------
      *
      * ------------------------------------
      *  Type Definitions
      * ------------------------------------
      /COPY QWSDL2RPG,TYPES
      *
     D timeValueShort_t...
     D                 DS                  qualified               based(pDummy)
     D  hours                         2A
     D  dot_1                         1A
     D  minutes                       2A
     D  dot_2                         1A
     D  seconds                       2A
      *
     D timeValueLong_t...
     D                 DS                  qualified               based(pDummy)
     D  hours                         2A
     D  dot_1                         1A
     D  minutes                       2A
     D  dot_2                         1A
     D  seconds                       2A
     D  dot_3                         1A
     D  mSecs                         3A
      *
     D attribute_t     DS                  qualified
     D  ns                                 like(wsdl_nsValue_t)
     D  name                               like(wsdl_lName_t  )
     D  value                              like(wsdl_value_t  )
      *
     D xmlTimestamp_t  DS                  qualified               based(pDummy)
     D  year                          4A                                        |
     D  minus_1                       1A                                        |
     D  month                         2A                                        |
     D  minus_2                       1A                                        |
     D  day                           2A                                        |
     D  char_T                        1A                                        | iSeries
     D  hour                          2A                                        | Timestamp
     D  colon_1                       1A                                        |
     D  minute                        2A                                        |
     D  colon_2                       1A                                        |
     D  second                        2A                                        |
     D  dot                           1A                                        |
     D  mSecs                         6A                                        |
     D  timeZone                           likeds(xmlTimeZone_t )
      *
     D xmlDate_t       DS                  qualified               based(pDummy)
     D  value                        10A
     D  timeZoneID                    1A
     D  hours                         2A
     D  delimiter                     1A
     D  minutes                       2A
      *
     D xmlTimeShort_t  DS                  qualified               based(pDummy)
     D  value                              likeds(timeValueShort_t)
     D  timeZone                           likeds(xmlTimeZone_t   )
      *
     D xmlTimeLong_t   DS                  qualified               based(pDummy)
     D  value                              likeds(timeValueLong_t )
     D  timeZone                           likeds(xmlTimeZone_t   )
      *
     D xmlTimeZone_t...
     D                 DS                  qualified               based(pDummy)
     D  ID                            1A
     D  hours                         2A
     D  delimiter                     1A
     D  minutes                       2A
      *
     D xmlGDay_t...
     D                 DS                  qualified               based(pDummy)
     D  prefix                        3A
     D  day                           2A
     D  timeZone                           likeds(xmlTimeZone_t   )
      *
     D xmlGMonth_t...
     D                 DS                  qualified               based(pDummy)
     D  prefix                        2A
     D  month                         2A
     D  delimiter                     2A
     D  timeZone                           likeds(xmlTimeZone_t   )
      *
     D xmlGYear_t...
     D                 DS                  qualified               based(pDummy)
     D  year                          4A
     D  timeZone                           likeds(xmlTimeZone_t   )
      *
     D xmlGMonthDay_t...
     D                 DS                  qualified               based(pDummy)
     D  prefix                        2A
     D  month                         2A
     D  delimiter                     1A
     D  day                           2A
     D  timeZone                           likeds(xmlTimeZone_t   )
      *
     D xmlGYearMonth_t...
     D                 DS                  qualified               based(pDummy)
     D  year                          4A
     D  delimiter                     1A
     D  month                         2A
     D  timeZone                           likeds(xmlTimeZone_t   )
      *
      *
     D utcOffset_t...
     D                 DS                  qualified               based(pDummy)
     D  timeZoneID                    1A
     D  hours                         2A
     D  minutes                       2A
      *
     D timestamp_t     DS                  qualified               based(pDummy)
     D  year                          4A                                        |
     D  minus_1                       1A                                        |
     D  month                         2A                                        |
     D  minus_2                       1A                                        |
     D  day                           2A                                        |
     D  char_T                        1A                                        | iSeries
     D  hour                          2A                                        | Timestamp
     D  colon_1                       1A                                        |
     D  minute                        2A                                        |
     D  colon_2                       1A                                        |
     D  second                        2A                                        |
     D  dot                           1A                                        |
     D  mSecs                         6A                                        |
     D  value                  1     26Z
      *
     D buffer_t        DS                  qualified               based(pDummy)
     D  field                              likeds(wsdl_fieldv6_t )
     D  size                         10I 0
     D  maxSize                      10I 0
     D  isV6                           N
     D  hMemWtr                            like(memWtr_handle_t  )
      *
      * ------------------------------------
      *  Exported prototypes
      * ------------------------------------
      /DEFINE INTERNALLY_COMPILE_WSDL2R98
      /COPY QWSDL2RPG,PWSDL2R98
      *
      * ------------------------------------
      *  Imported prototypes
      * ------------------------------------
      /COPY QWSDL2RPG,CLIB                         C Runtime Library
      *
      /COPY QBASICS1,PBASICS1                      Tools/400 Runtime Library
      /COPY QWSDL2RPG,BASE64_H                     Base64 Encode/Decoder by Scott Klement
      /COPY QWSDL2RPG,PWSDL2R81                    WSDL2RPG: Character Translation
      /COPY QWSDL2RPG,PWSDL2R87                    WSDL2RPG: ManagedMemoryDataSource
      /COPY QWSDL2RPG,PWSDL2R88                    WSDL2RPG: Public Helper Procedures
      /COPY QWSDL2RPG,PWSDL2R90                    WSDL2RPG: Application Message
      /COPY QWSDL2RPG,PWSDL2R97                    WSDL2RPG: eXpat Adapter
      *
      * ------------------------------------
      *  Internal prototypes
      * ------------------------------------
      *
      *  Checks whether or not the XML encoder is enabled
      *  end encodes the XML entities of a given string.
     D encodeXmlEntityToFieldV6...
     D                 PR
     D                                     extproc('+
     D                                     encodeXmlEntityToFieldV6+
     D                                     ')
     D  i_fromField                        const  likeds(wsdl_fieldv6_t )
     D  i_toField                          const  likeds(wsdl_fieldv6_t )
      *
      *  Checks whether or not the XML encoder is enabled
      *  end encodes the XML entities of a given string.
     D encodeXmlEntityToStream...
     D                 PR            10I 0
     D                                     extproc('+
     D                                     encodeXmlEntityToStream+
     D                                     ')
     D  i_fromField                        const  likeds(wsdl_fieldv6_t )
     D  i_fd                               const  like(memWtr_handle_t  )
     D  i_pWriteProc                   *   const  procptr
      *
      *  Encodes an XML entities.
     D encodeXmlEntityToStreamInternally...
     D                 PR            10I 0
     D                                     extproc('+
     D                                     encodeXmlEntityToStreamInternally+
     D                                     ')
     D  i_fromField                        const  likeds(wsdl_fieldv6_t )
     D  i_fd                               const  like(memWtr_handle_t  )
     D  i_pWriteProc                   *   const  procptr
      *
      *  Writes to a generic 'write' procedure.
     D GenericStream_write...
     D                 PR                         like(ssize_t   )
     D                                     extproc('+
     D                                     GenericStream_write+
     D                                     ')
     D  i_fd                         10I 0 const                                = int
     D  i_pWriteProc                   *   const  procptr
     D  i_pBuffer                      *   const                                = *buf
     D  i_buflen                           const  like(size_t    )              = size_t
      *
      *  Returns the length of a varying field
     D getLengthV6...
     D                 PR            10U 0
     D                                     extproc('+
     D                                     getLengthV6+
     D                                     ')
     D  i_field                            const  likeds(wsdl_fieldv6_t )
      *
     D getMaxLengthV6...
     D                 PR            10U 0
     D                                     extproc('+
     D                                     getMaxLengthV6+
     D                                     ')
     D  i_field                            const  likeds(wsdl_fieldv6_t )
      *
      *  Sets the length of a given varying field
     D setLengthV6...
     D                 PR
     D                                     extproc('+
     D                                     setLengthV6+
     D                                     ')
     D  i_field                            const  likeds(wsdl_fieldv6_t )
     D  i_length                     10U 0 const
      *
      *  Returns cTrue for BigString fields.
     D isBigStringV6...
     D                 PR              N
     D                                     extproc('+
     D                                     isBigStringV6+
     D                                     ')
     D  i_field                            const  likeds(wsdl_fieldv6_t )
      *
      *  Returns cTrue for VARYING fields.
     D isVaryingV6...
     D                 PR              N
     D                                     extproc('+
     D                                     isVaryingV6+
     D                                     ')
     D  i_field                            const  likeds(wsdl_fieldv6_t )
     D  o_msg                                     likeds(msg_t          )
      *
      *  Returns cTrue if the specified field is too short
      *  to store a value of the specified length.
     D isTooShortV6...
     D                 PR              N
     D                                     extproc('+
     D                                     isTooShortV6+
     D                                     ')
     D  i_field                            const  likeds(wsdl_fieldv6_t )
     D  i_length                     10I 0 const
     D  o_msg                                     likeds(msg_t          )
      *
      *  Returns the name of the current field.
     D getFieldNameV6...
     D                 PR          1024A          varying
     D                                     extproc('+
     D                                     getFieldNameV6+
     D                                     ')
      *
      *  Copies a given buffer into a given field.
     D copyValueV6...
     D                 PR              N
     D                                     extproc('+
     D                                     copyValueV6+
     D                                     ')
     D  i_field                            const  likeds(wsdl_fieldv6_t )
     D  i_pValue                       *   const
     D  i_length                     10I 0 const
     D  o_msg                                     likeds(msg_t          )
      *
      * Appends a given string to a given buffer of type 'schema' or 'any'.
     D Buffer_append...
     D                 PR
     D                                     extproc('+
     D                                     Buffer_append+
     D                                     ')
     D  io_buffer                                   likeds(buffer_t )
     D  i_string                       *   const    options(*string)
      *
      * Replaces a given string within the specified buffer.
     D Buffer_replace...
     D                 PR
     D                                     extproc('+
     D                                     Buffer_replace+
     D                                     ')
     D  io_buffer                                   likeds(buffer_t )
     D  i_arg                          *   const    options(*string)
     D  i_value                        *   const    options(*string)
      *
      * Terminates 'schema' processing and returns
      * the schema to the caller.
     D Schema_terminate...
     D                 PR
     D                                     extproc('WSDL2R98_+
     D                                     Schema_terminate+
     D                                     ')
      *
      * Terminates 'any' processing and returns
      * the schema to the caller.
     D Any_terminate...
     D                 PR
     D                                     extproc('WSDL2R98_+
     D                                     Any_terminate+
     D                                     ')
      *
      *  Adjust remote timestamp to local time zone.
     D adjustTimestamp...
     D                 PR              Z
     D                                     extproc('WSDL2R98_+
     D                                     adjustTimestamp+
     D                                     ')
     D  i_xmlTimestamp...
     D                                     const  likeds(xmlTimestamp_t )
      *
      *  Adjust remote date to local time zone.
     D adjustDate...
     D                 PR              D
     D                                     extproc('WSDL2R98_+
     D                                     adjustDate+
     D                                     ')
     D  i_xmlDate                          const  likeds(xmlDate_t )
      *
      *  Adjust remote time to local time zone.
     D adjustTime...
     D                 PR              T
     D                                     extproc('WSDL2R98_+
     D                                     adjustTime+
     D                                     ')
     D  i_xmlTime                          const  likeds(xmlTimeShort_t)
      *
      *  Adjust remote gDay to local time zone.
     D adjustGDay...
     D                 PR                         like(wsdl_gDay_t      )
     D                                     extproc('WSDL2R98_+
     D                                     adjustGDay+
     D                                     ')
     D  i_xmlGDay                          const  likeds(xmlGDay_t      )
      *
      *  Adjust remote gMonth to local time zone.
     D adjustGMonth...
     D                 PR                         like(wsdl_gMonth_t    )
     D                                     extproc('WSDL2R98_+
     D                                     adjustGMonth+
     D                                     ')
     D  i_xmlGMonth                        const  likeds(xmlGMonth_t    )
      *
      *  Adjust remote gMonthDay to local time zone.
     D adjustGMonthDay...
     D                 PR                         like(wsdl_gMonthDay_t )
     D                                     extproc('WSDL2R98_+
     D                                     adjustGMonthDay+
     D                                     ')
     D  i_xmlGMonthDay...
     D                                     const  likeds(xmlGMonthDay_t )
      *
      *  Adjust remote gYear to local time zone.
     D adjustGYear...
     D                 PR                         like(wsdl_gYear_t     )
     D                                     extproc('WSDL2R98_+
     D                                     adjustGYear+
     D                                     ')
     D  i_xmlGYear                         const  likeds(xmlGYear_t     )
      *
      *  Adjust remote gYearMonth to local time zone.
     D adjustGYearMonth...
     D                 PR                         like(wsdl_gYearMonth_t)
     D                                     extproc('WSDL2R98_+
     D                                     adjustGYearMonth+
     D                                     ')
     D  i_xmlGYearMonth...
     D                                     const  likeds(xmlGYearMonth_t)
      *
      *  Merge remote hours and minutes with local time zone.
     D mergeWithLocalTimeOffset...
     D                 PR              Z
     D                                     extproc('WSDL2R98_+
     D                                     mergeWithLocalTimeOffset+
     D                                     ')
     D  i_timestamp                    Z   const
     D  i_hours                       5I 0 const
     D  i_minutes                     5I 0 const
      *
      *  Parses a given XML time value.
     D parseXmlTime...
     D                 PR                         likeds(xmlTimeShort_t)
     D                                     extproc('WSDL2R98_+
     D                                     parseXmlTime+
     D                                     ')
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Returns the time zone value based on the input time zone.
     D getXmlTimeZone...
     D                 PR                         likeds(xmlTimeZone_t )
     D                                     extproc('WSDL2R98_+
     D                                     getXmlTimeZone+
     D                                     ')
     D  i_timeZone                    6A   const  options(*nopass)
      *
      *  Returns TRUE if a given character indicates the start of the
      *  timezone portion of a date/time value.
     D isStartOfTimezone...
     D                 PR              N
     D                                     extproc('WSDL2R98_+
     D                                     isStartOfTimezone+
     D                                     ')
     D  i_char                        1A   const
      *
      *  Returns TRUE if a given zimezone matches the specification.
     D isValidTimezone...
     D                 PR              N
     D                                     extproc('WSDL2R98_+
     D                                     isValidTimezone+
     D                                     ')
     D  i_timeZone                         const  likeds(xmlTimeZone_t )
      *
      *  Edits a decimal value removing trailing zeros.
     D editDecimal...
     D                 PR            64A          varying
     D                                     extproc('WSDL2R98_+
     D                                     editDecimal+
     D                                     ')
     D  i_decimal                          const  like(wsdl_decimal_t )
      *
      *  Returns the default 'write' procedure used to write marshalled data.
     D getDefaultWriteProc...
     D                 PR              *          procptr
     D                                     extproc('WSDL2R98_+
     D                                     getDefaultWriteProc+
     D                                     ')
      *
      *  Returns cFalse if a given character does not apply to the rules
      *  of a normalized string.
     D isNormalizedStringChar...
     D                 PR              N
     D                                     extproc('WSDL2R98_+
     D                                     isNormalizedStringChar+
     D                                     ')
     D  i_char                        1A   const
      *
      *  Clears the name and xpath of the field that has been unmarshalled.
     D clearFieldIdentifier...
     D                 PR
     D                                     extproc('WSDL2R98_+
     D                                     clearFieldIdentifier+
     D                                     ')
      *
      *  Produces a message to signal an unexpected unmarshaller error.
     D produceUnexpectedErrorMessage...
     D                 PR                         likeds(msg_t          )
     D                                     extproc('WSDL2R98_+
     D                                     produceUnexpectedErrorMessage+
     D                                     ')
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Produces a message to signal an unexpected unmarshaller error.
     D produceUnexpectedErrorMessageV6...
     D                 PR                         likeds(msg_t          )
     D                                     extproc('WSDL2R98_+
     D                                     produceUnexpectedErrorMessageV6+
     D                                     ')
     D  i_pValue                       *   const
     D  i_length                     10I 0 value
      *
      *  Produces a message to signal an unexpected unmarshaller error.
     D produceUnexpectedErrorMessageIntern...
     D                 PR                         likeds(msg_t          )
     D                                     extproc('WSDL2R98_+
     D                                     produceUnexpectedErrorMessageIntern+
     D                                     ')
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      * ------------------------------------
      *  Global fields & constants
      * ------------------------------------
     D g_doEncodeXmlEntities...
     D                 S               N   inz(cTrue)
      *
     D g_ns            S                   like(wsdl_nsValue_t) dim(32)
      *
     D g_schema        DS                  likeds(buffer_t ) inz
     D g_any           DS                  likeds(buffer_t ) inz
      *
     D g_fieldIdentifier...
     D                 DS                  qualified
     D  isSet                          N   inz(cFalse)
     D  name                       1024A   varying inz('*N')
     D  path                      24576A   varying inz('*N')
     D  index                        10I 0 inz(-1)
      *
     D cNS_PREFIX      C                   const('tns')
     D cNS_XSD         C                   const('xsd')
      *
      *=====================================================================*
      *  Sets the name and xpath of the field that is going                 *
      *  to be unmarshalled.                                                *
      *=====================================================================*
     P UnMarshaller_setFieldIdentifier...
     P                 B                   export
      *
     D UnMarshaller_setFieldIdentifier...
     D                 PI
     D  i_name                     1024A   const  varying options(*varsize)
     D  i_path                    24576A   const  varying options(*varsize)
     D  i_index                      10I 0 const  options(*nopass)
      *
      *  Parameter positions
     D p_index         C                   3
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         g_fieldIdentifier.isSet = cTrue;
         g_fieldIdentifier.name  = %trimR(i_name);
         g_fieldIdentifier.path  = %trimR(i_path);

         if (%parms() >= p_index);
            g_fieldIdentifier.index = i_index;
         else;
            reset g_fieldIdentifier.index;
         endif;

         return;

      /END-FREE
      *
     P UnMarshaller_setFieldIdentifier...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'anyType' (actually string).
      *=====================================================================*
     P UnMarshaller_toAnyType...
     P                 B                   export
      *
     D UnMarshaller_toAnyType...
     D                 PI         65535A          varying
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (%len(i_value) > 0);
               return %subst(i_value: 1: %len(i_value));
            else;
               return '';
            endif;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(produceUnexpectedErrorMessage(i_value));
         endmon;

      /END-FREE
      *
     P UnMarshaller_toAnyType...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'anyType' (actually string).
      *=====================================================================*
     P UnMarshaller_toAnyTypeV6...
     P                 B                   export
      *
     D UnMarshaller_toAnyTypeV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_size                       10I 0 const
     D  i_pValue                       *   const
     D  i_length                     10I 0 const
      *
      *  Return value
      *
      *  Helper fields
     D toField         DS                  likeds(wsdl_fieldv6_t ) inz
      *
      *  Local fields
     D msg             DS                  likeds(msg_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         toField.pField = i_pField;
         toField.pData  = i_pData;
         toField.size   = i_size;

         monitor;
            // Check 'varying' property
            if (not isVaryingV6(toField: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Check maximum length of data of target field
            if (isTooShortV6(toField: i_length: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Copy data to field
            if (not copyValueV6(toField: i_pValue: i_length: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Set length of copied data
            setLengthV6(toField: i_length);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessageV6(i_pValue: i_length));
         endmon;

      /END-FREE
      *
     P UnMarshaller_toAnyTypeV6...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'anySimpleType' (actually string).
      *=====================================================================*
     P UnMarshaller_toAnySimpleType...
     P                 B                   export
      *
     D UnMarshaller_toAnySimpleType...
     D                 PI         65535A          varying
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (%len(i_value) > 0);
               return %subst(i_value: 1: %len(i_value));
            else;
               return '';
            endif;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(produceUnexpectedErrorMessage(i_value));
         endmon;

      /END-FREE
      *
     P UnMarshaller_toAnySimpleType...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'anySimpleType' (actually string).
      *=====================================================================*
     P UnMarshaller_toAnySimpleTypeV6...
     P                 B                   export
      *
     D UnMarshaller_toAnySimpleTypeV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_size                       10I 0 const
     D  i_pValue                       *   const
     D  i_length                     10I 0 const
      *
      *  Return value
      *
      *  Helper fields
     D toField         DS                  likeds(wsdl_fieldv6_t ) inz
      *
      *  Local fields
     D msg             DS                  likeds(msg_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         toField.pField = i_pField;
         toField.pData  = i_pData;
         toField.size   = i_size;

         monitor;
            // Check 'varying' property
            if (not isVaryingV6(toField: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Check maximum length of data of target field
            if (isTooShortV6(toField: i_length: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Copy data to field
            if (not copyValueV6(toField: i_pValue: i_length: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Set length of copied data
            setLengthV6(toField: i_length);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessageV6(i_pValue: i_length));
         endmon;

      /END-FREE
      *
     P UnMarshaller_toAnySimpleTypeV6...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'any'     (actually string).
      *=====================================================================*
     P UnMarshaller_toAny...
     P                 B                   export
      *
     D UnMarshaller_toAny...
     D                 PI         65535A          varying
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (%len(i_value) > 0);
               return %subst(i_value: 1: %len(i_value));
            else;
               return '';
            endif;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(produceUnexpectedErrorMessage(i_value));
         endmon;

      /END-FREE
      *
     P UnMarshaller_toAny...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'any'     (actually string).
      *=====================================================================*
     P UnMarshaller_toAnyV6...
     P                 B                   export
      *
     D UnMarshaller_toAnyV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_size                       10I 0 const
     D  i_pValue                       *   const
     D  i_length                     10I 0 const
      *
      *  Return value
      *
      *  Helper fields
     D toField         DS                  likeds(wsdl_fieldv6_t ) inz
      *
      *  Local fields
     D msg             DS                  likeds(msg_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         toField.pField = i_pField;
         toField.pData  = i_pData;
         toField.size   = i_size;

         monitor;
            // Check 'varying' property
            if (not isVaryingV6(toField: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Check maximum length of data of target field
            if (isTooShortV6(toField: i_length: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Copy data to field
            if (not copyValueV6(toField: i_pValue: i_length: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Set length of copied data
            setLengthV6(toField: i_length);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessageV6(i_pValue: i_length));
         endmon;

      /END-FREE
      *
     P UnMarshaller_toAnyV6...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'string'.
      *=====================================================================*
     P UnMarshaller_toString...
     P                 B                   export
      *
     D UnMarshaller_toString...
     D                 PI         65535A          varying
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (%len(i_value) > 0);
               return %subst(i_value: 1: %len(i_value));
            else;
               return '';
            endif;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

      /END-FREE
      *
     P UnMarshaller_toString...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'string'.
      *=====================================================================*
     P UnMarshaller_toStringV6...
     P                 B                   export
      *
     D UnMarshaller_toStringV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_size                       10I 0 const
     D  i_pValue                       *   const
     D  i_length                     10I 0 const
      *
      *  Return value
      *
      *  Helper fields
     D toField         DS                  likeds(wsdl_fieldv6_t ) inz
      *
      *  Local fields
     D msg             DS                  likeds(msg_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         toField.pField = i_pField;
         toField.pData  = i_pData;
         toField.size   = i_size;

         monitor;
            // Check 'varying' property
            if (not isVaryingV6(toField: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Check maximum length of data of target field
            if (isTooShortV6(toField: i_length: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Copy data to field
            if (not copyValueV6(toField: i_pValue: i_length: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Set length of copied data
            setLengthV6(toField: i_length);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessageV6(i_pValue: i_length));
         endmon;

      /END-FREE
      *
     P UnMarshaller_toStringV6...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'normalizedString'.
      *=====================================================================*
     P UnMarshaller_toNormalizedString...
     P                 B                   export
      *
     D UnMarshaller_toNormalizedString...
     D                 PI         65535A          varying
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            if (%len(i_value) > 0);
               return %subst(i_value: 1: %len(i_value));
            else;
               return '';
            endif;
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

      /END-FREE
      *
     P UnMarshaller_toNormalizedString...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'normalizedString'.
      *=====================================================================*
     P UnMarshaller_toNormalizedStringV6...
     P                 B                   export
      *
     D UnMarshaller_toNormalizedStringV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_size                       10I 0 const
     D  i_pValue                       *   const
     D  i_length                     10I 0 const
      *
      *  Return value
      *
      *  Helper fields
     D toField         DS                  likeds(wsdl_fieldv6_t ) inz
      *
      *  Local fields
     D msg             DS                  likeds(msg_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         toField.pField = i_pField;
         toField.pData  = i_pData;
         toField.size   = i_size;

         monitor;
            // Check 'varying' property
            if (not isVaryingV6(toField: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Check maximum length of data of target field
            if (isTooShortV6(toField: i_length: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Copy data to field
            if (not copyValueV6(toField: i_pValue: i_length: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Set length of copied data
            setLengthV6(toField: i_length);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessageV6(i_pValue: i_length));
         endmon;

      /END-FREE
      *
     P UnMarshaller_toNormalizedStringV6...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'QName'.
      *=====================================================================*
     P UnMarshaller_toQName...
     P                 B                   export
      *
     D UnMarshaller_toQName...
     D                 PI         65535A          varying
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            if (%len(i_value) > 0);
               return %subst(i_value: 1: %len(i_value));
            else;
               return '';
            endif;
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

      /END-FREE
      *
     P UnMarshaller_toQName...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'QName'.
      *=====================================================================*
     P UnMarshaller_toQNameV6...
     P                 B                   export
      *
     D UnMarshaller_toQNameV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_size                       10I 0 const
     D  i_pValue                       *   const
     D  i_length                     10I 0 const
      *
      *  Return value
      *
      *  Helper fields
     D toField         DS                  likeds(wsdl_fieldv6_t ) inz
      *
      *  Local fields
     D msg             DS                  likeds(msg_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         toField.pField = i_pField;
         toField.pData  = i_pData;
         toField.size   = i_size;

         monitor;
            // Check 'varying' property
            if (not isVaryingV6(toField: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Check maximum length of data of target field
            if (isTooShortV6(toField: i_length: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Copy data to field
            if (not copyValueV6(toField: i_pValue: i_length: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Set length of copied data
            setLengthV6(toField: i_length);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessageV6(i_pValue: i_length));
         endmon;

      /END-FREE
      *
     P UnMarshaller_toQNameV6...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'boollean'.
      *=====================================================================*
     P UnMarshaller_toBoolean...
     P                 B                   export
      *
     D UnMarshaller_toBoolean...
     D                 PI              N
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D boolean         S               N   inz
      *
      *  Messages
      /COPY QWSDL2RPG,USR5000                     Failed to convert boolean value '&1' to RPG.
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            select;
            when (i_value = 'true' or i_value = '1');
               boolean = cTrue;
            when (i_value = 'false' or i_value = '0');
               boolean = cFalse;
            other;
               clear USR5000;
               USR50001 = i_value;
               AppMsg_sendEscMsg(AppMsg_new('USR5000': USR5000));
            endsl;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return boolean;

      /END-FREE
      *
     P UnMarshaller_toBoolean...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'decimal'.
      *=====================================================================*
     P UnMarshaller_toDecimal...
     P                 B                   export
      *
     D UnMarshaller_toDecimal...
     D                 PI                         like(wsdl_decimal_t )
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D decimal         S                   like(wsdl_decimal_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            decimal = %dec(i_value: cDECIMAL_LEN: cDECIMAL_DECPOS);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return decimal;

      /END-FREE
      *
     P UnMarshaller_toDecimal...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'float'.
      *=====================================================================*
     P UnMarshaller_toFloat...
     P                 B                   export
      *
     D UnMarshaller_toFloat...
     D                 PI                         like(wsdl_float_t   )
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D float           S                   like(wsdl_float_t   ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            float = %dec(i_value: cFLOAT_LEN: cFLOAT_DECPOS);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return float;

      /END-FREE
      *
     P UnMarshaller_toFloat...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'double'.
      *=====================================================================*
     P UnMarshaller_toDouble...
     P                 B                   export
      *
     D UnMarshaller_toDouble...
     D                 PI                         like(wsdl_double_t  )
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D double          S                   like(wsdl_double_t  ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            double = %dec(i_value: cDOUBLE_LEN: cDOUBLE_DECPOS);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return double;

      /END-FREE
      *
     P UnMarshaller_toDouble...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'short'.
      *=====================================================================*
     P UnMarshaller_toShort...
     P                 B                   export
      *
     D UnMarshaller_toShort...
     D                 PI                  like(wsdl_short_t )
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D short           S                   like(wsdl_short_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            short = %dec(i_value: cSHORT_LEN: cSHORT_DECPOS);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return short;

      /END-FREE
      *
     P UnMarshaller_toShort...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'byte'.
      *=====================================================================*
     P UnMarshaller_toByte...
     P                 B                   export
      *
     D UnMarshaller_toByte...
     D                 PI                  like(wsdl_byte_t  )
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D byte            S                   like(wsdl_byte_t  ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            byte = %dec(i_value: cBYTE_LEN: cBYTE_DECPOS);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return byte;

      /END-FREE
      *
     P UnMarshaller_toByte...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'integer'.
      *=====================================================================*
     P UnMarshaller_toInt...
     P                 B                   export
      *
     D UnMarshaller_toInt...
     D                 PI                  like(wsdl_int_t     )
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D int             S                   like(wsdl_int_t     ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            int = %dec(i_value: cINT_LEN: cINT_DECPOS);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return int;

      /END-FREE
      *
     P UnMarshaller_toInt...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'integer'.
      *=====================================================================*
     P UnMarshaller_toInteger...
     P                 B                   export
      *
     D UnMarshaller_toInteger...
     D                 PI                  like(wsdl_integer_t )
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D integer         S                   like(wsdl_integer_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            integer = %dec(i_value: cINTEGER_LEN: cINTEGER_DECPOS);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return integer;

      /END-FREE
      *
     P UnMarshaller_toInteger...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'long'.
      *=====================================================================*
     P UnMarshaller_toLong...
     P                 B                   export
      *
     D UnMarshaller_toLong...
     D                 PI                  like(wsdl_long_t    )
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D long            S                   like(wsdl_long_t    ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            long = %dec(i_value: cLONG_LEN: cLONG_DECPOS);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return long;

      /END-FREE
      *
     P UnMarshaller_toLong...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'unsigned short'.
      *=====================================================================*
     P UnMarshaller_toUnsignedShort...
     P                 B                   export
      *
     D UnMarshaller_toUnsignedShort...
     D                 PI                  like(wsdl_uShort_t)
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D uShort          S                   like(wsdl_uShort_t) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            uShort = %dec(i_value: cUSHORT_LEN: cUSHORT_DECPOS);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return uShort;

      /END-FREE
      *
     P UnMarshaller_toUnsignedShort...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'unsigned byte'.
      *=====================================================================*
     P UnMarshaller_toUnsignedByte...
     P                 B                   export
      *
     D UnMarshaller_toUnsignedByte...
     D                 PI                  like(wsdl_uByte_t )
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D uByte           S                   like(wsdl_uByte_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            uByte = %dec(i_value: cUBYTE_LEN: cUBYTE_DECPOS);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return uByte;

      /END-FREE
      *
     P UnMarshaller_toUnsignedByte...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'unsigned integer'.
      *=====================================================================*
     P UnMarshaller_toUnsignedInt...
     P                 B                   export
      *
     D UnMarshaller_toUnsignedInt...
     D                 PI                  like(wsdl_uInt_t    )
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D uInt            S                   like(wsdl_uInt_t    ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            uInt = %dec(i_value: cUINT_LEN: cUINT_DECPOS);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return uInt;

      /END-FREE
      *
     P UnMarshaller_toUnsignedInt...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'unsigned long'.
      *=====================================================================*
     P UnMarshaller_toUnsignedLong...
     P                 B                   export
      *
     D UnMarshaller_toUnsignedLong...
     D                 PI                  like(wsdl_uLong_t   )
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D uLong           S                   like(wsdl_uLong_t   ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            uLong = %dec(i_value: cULONG_LEN: cULONG_DECPOS);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return uLong;

      /END-FREE
      *
     P UnMarshaller_toUnsignedLong...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'non negative integer'.
      *=====================================================================*
     P UnMarshaller_toNonNegativeInteger...
     P                 B                   export
      *
     D UnMarshaller_toNonNegativeInteger...
     D                 PI                  like(wsdl_nonNegInt_t )
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D nonNegInt       S                   like(wsdl_nonNegInt_t ) inz
      *
      *  Messages
      /COPY QWSDL2RPG,USR5003
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            nonNegInt = %dec(i_value: cNON_NEG_INT_LEN: cNON_NEG_INT_DECPOS);

            if (nonNegInt < 0);
               clear USR5003;
               USR50031 = %char(nonNegInt);
               USR50032 = 1;   // negative values are not allowed
               AppMsg_sendEscMsg(AppMsg_newError('USR5003': USR5003));
            endif;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return nonNegInt;

      /END-FREE
      *
     P UnMarshaller_toNonNegativeInteger...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'non positive integer'.
      *=====================================================================*
     P UnMarshaller_toNonPositiveInteger...
     P                 B                   export
      *
     D UnMarshaller_toNonPositiveInteger...
     D                 PI                  like(wsdl_nonPosInt_t )
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D nonPosInt       S                   like(wsdl_nonPosInt_t ) inz
      *
      *  Messages
      /COPY QWSDL2RPG,USR5003
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            nonPosInt = %dec(i_value: cNON_POS_INT_LEN: cNON_POS_INT_DECPOS);

            if (nonPosInt > 0);
               clear USR5003;
               USR50031 = %char(nonPosInt);
               USR50032 = 2;   // positive values are not allowed
               AppMsg_sendEscMsg(AppMsg_newError('USR5003': USR5003));
            endif;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return nonPosInt;

      /END-FREE
      *
     P UnMarshaller_toNonPositiveInteger...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'negative integer'.
      *=====================================================================*
     P UnMarshaller_toNegativeInteger...
     P                 B                   export
      *
     D UnMarshaller_toNegativeInteger...
     D                 PI                  like(wsdl_negInt_t )
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D negInt          S                   like(wsdl_negInt_t ) inz
      *
      *  Messages
      /COPY QWSDL2RPG,USR5003
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            negInt = %dec(i_value: cNEG_INT_LEN: cNEG_INT_DECPOS);

            if (negInt >= 0);
               clear USR5003;
               USR50031 = %char(negInt);
               USR50032 = 3;   // positive values and 0 are not allowed
               AppMsg_sendEscMsg(AppMsg_newError('USR5003': USR5003));
            endif;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return negInt;

      /END-FREE
      *
     P UnMarshaller_toNegativeInteger...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'positive integer'.
      *=====================================================================*
     P UnMarshaller_toPositiveInteger...
     P                 B                   export
      *
     D UnMarshaller_toPositiveInteger...
     D                 PI                  like(wsdl_posInt_t )
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D posInt          S                   like(wsdl_posInt_t ) inz
      *
      *  Messages
      /COPY QWSDL2RPG,USR5003
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            posInt = %dec(i_value: cPOS_INT_LEN: cPOS_INT_DECPOS);

            if (posInt <= 0);
               clear USR5003;
               USR50031 = %char(posInt);
               USR50032 = 4;   // negative values and 0 are not allowed
               AppMsg_sendEscMsg(AppMsg_newError('USR5003': USR5003));
            endif;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return posInt;

      /END-FREE
      *
     P UnMarshaller_toPositiveInteger...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'timestamp'.
      *=====================================================================*
      *  Sample java.util.Date value:   2007-01-11T23:00:00.000Z
      *  Sample RPG timestamp value :   2007-01-11-23.00.00.000000
      *=====================================================================*
     P UnMarshaller_toDateTime...
     P                 B                   export
      *
     D UnMarshaller_toDateTime...
     D                 PI              Z
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D timestamp       S               Z   inz
      *
      *  Helper fields
     D x               S             10I 0 inz
     D char            S              1A   inz
     D inTimestamp     DS                  likeds(xmlTimestamp_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            x = 1;
            dow (x <= %len(i_value) and x <= %len(inTimestamp));
               char = %subst(i_value: x: 1);
               if (x >= 20 and isStartOfTimezone(char));
                  inTimestamp.timeZone = char;
                  leave;
               endif;
               %subst(inTimestamp: x: 1) = char;
               x = x + 1;
            enddo;

            inTimestamp.dot = '.';   // Dot between time and microseconds

            // Adjust timestamp with respect to the local time zone.
            select;
            when (inTimestamp.timeZone.ID = 'Z');        // UTC time
               timestamp = adjustTimestamp(inTimestamp);
            when (inTimestamp.timeZone.ID = '-');        // negative time offset
               timestamp = adjustTimestamp(inTimestamp);
            when (inTimestamp.timeZone.ID = '+');        // negative time offset
               timestamp = adjustTimestamp(inTimestamp);
            other;
               timestamp = adjustTimestamp(inTimestamp); // no time zone inf. available
            endsl;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return timestamp;

      /END-FREE
      *
     P UnMarshaller_toDateTime...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'date'.
      *=====================================================================*
      *  Sample XML date values:   2007-01-11
      *                            2007-01-11Z        UTC time zone
      *                            2007-01-11-06:00   offset from UTC time
      *                            2007-01-11+06:00   offset from UTC time
      *=====================================================================*
     P UnMarshaller_toDate...
     P                 B                   export
      *
     D UnMarshaller_toDate...
     D                 PI              D
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D date            S               D   inz
      *
      *  Helper fields
     D inDate          DS                  likeds(xmlDate_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            clear inDate;
            inDate = i_value;

            // Adjust date with respect to the local time zone.
            select;
            when (inDate.timeZoneID = 'Z');   // UTC time
               date = adjustDate(inDate);
            when (inDate.timeZoneID = '-');   // negative time offset
               date = adjustDate(inDate);
            when (inDate.timeZoneID = '+');   // negative time offset
               date = adjustDate(inDate);
            other;
               date = %date(inDate.value);    // no time zone inf. available
            endsl;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return date;

      /END-FREE
      *
     P UnMarshaller_toDate...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'time'.
      *=====================================================================*
      *  Sample XML time values:   09:00:00
      *                            09:30:10Z          UTC time zone
      *                            09:30:10-06:00     offset from UTC time
      *                            09:30:10+06:00     offset from UTC time
      *=====================================================================*
     P UnMarshaller_toTime...
     P                 B                   export
      *
     D UnMarshaller_toTime...
     D                 PI              T
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D time            S               T   inz
      *
      *  Helper fields
     D inTimeShort     DS                  likeds(xmlTimeShort_t  ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            inTimeShort = parseXmlTime(i_value);

            // Adjust time with respect to the local time zone.
            select;
            when (inTimeShort.timeZone.ID = 'Z');
               time = adjustTime(inTimeShort);
            when (inTimeShort.timeZone.ID = '-');    // negative time offset
               time = adjustTime(inTimeShort);
            when (inTimeShort.timeZone.ID = '+');    // negative time offset
               time = adjustTime(inTimeShort);
            other;
               time = %time(inTimeShort.value);     // no time zone inf. available
            endsl;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return time;

      /END-FREE
      *
     P UnMarshaller_toTime...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'gDay'.
      *=====================================================================*
      *  Sample XML date values:   ---18
      *                            ---18Z             UTC time zone
      *                            ---18-06:00        offset from UTC time
      *                            ---18+06:00        offset from UTC time
      *=====================================================================*
     P UnMarshaller_togDay...
     P                 B                   export
      *
     D UnMarshaller_togDay...
     D                 PI                         like(wsdl_gDay_t       )
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D gDay            S                   like(wsdl_gDay_t       ) inz
      *
      *  Helper fields
     D inDay           DS                  likeds(xmlGDay_t   ) inz
      *
      *  Messages
      /COPY QWSDL2RPG,USR5012                      Invalid XML gDay value: '&1'
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            clear inDay;
            inDay = i_value;

            if (not isValidTimezone(inDay.timezone) or
                inDay.prefix <> *ALL'-');
               clear USR5012;
               USR50121 = i_value;
               AppMsg_sendEscMsg(AppMsg_newError('USR5012': USR5012));
            endif;

            // Adjust day with respect to the local time zone.
            select;
            when (inDay.timeZone.ID = 'Z');    // UTC time
               gDay = adjustGDay(inDay);
            when (inDay.timeZone.ID = '-');    // negative time offset
               gDay = adjustGDay(inDay);
            when (inDay.timeZone.ID = '+');    // negative time offset
               gDay = adjustGDay(inDay);
            other;
               gDay = %dec(inDay.day: cGDAY_LEN: cGDAY_DECPOS);
            endsl;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return gDay;

      /END-FREE
      *
     P UnMarshaller_togDay...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'gMonth'.
      *=====================================================================*
      *  Sample XML date values:   --06--
      *                            --06--Z            UTC time zone
      *                            --06---06:00       offset from UTC time
      *                            --06--+06:00       offset from UTC time
      *=====================================================================*
     P UnMarshaller_togMonth...
     P                 B                   export
      *
     D UnMarshaller_togMonth...
     D                 PI                         like(wsdl_gMonth_t     )
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D gMonth          S                   like(wsdl_gMonth_t     ) inz
      *
      *  Helper fields
     D inMonth         DS                  likeds(xmlGMonth_t ) inz
      *
      *  Messages
      /COPY QWSDL2RPG,USR5013                      Invalid XML gMonth value: '&1'
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            clear inMonth;
            inMonth = i_value;

            if (not isValidTimezone(inMonth.timezone) or
                inMonth.prefix <> *ALL'-' or
                inMonth.delimiter <> *ALL'-');
               clear USR5013;
               USR50131 = i_value;
               AppMsg_sendEscMsg(AppMsg_newError('USR5013': USR5013));
            endif;

            // Adjust month with respect to the local time zone.
            select;
            when (inMonth.timeZone.ID = 'Z');    // UTC time
               gMonth = adjustGMonth(inMonth);
            when (inMonth.timeZone.ID = '-');    // negative time offset
               gMonth = adjustGMonth(inMonth);
            when (inMonth.timeZone.ID = '+');    // negative time offset
               gMonth = adjustGMonth(inMonth);
            other;
               gMonth = %dec(inMonth.month: cGDAY_LEN: cGDAY_DECPOS);
            endsl;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return gMonth;

      /END-FREE
      *
     P UnMarshaller_togMonth...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'gMonthDay'.
      *=====================================================================*
      *  Sample XML date values:   --06-18
      *                            --06-18Z           UTC time zone
      *                            --06-18-06:00      offset from UTC time
      *                            --06-18+06:00      offset from UTC time
      *=====================================================================*
     P UnMarshaller_togMonthDay...
     P                 B                   export
      *
     D UnMarshaller_togMonthDay...
     D                 PI                         like(wsdl_gMonthDay_t  )
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D gMonthDay       S                   like(wsdl_gMonthDay_t  ) inz
      *
      *  Helper fields
     D inGMonthDay     DS                  likeds(xmlGMonthDay_t ) inz
     D month           S              5I 0 inz
     D day             S              5I 0 inz
      *
      *  Messages
      /COPY QWSDL2RPG,USR5010                     Invalid XML gMonthDay value: '&1'
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            clear inGMonthDay;
            inGMonthDay = i_value;

            if (not isValidTimezone(inGMonthDay.timezone) or
                inGMonthDay.prefix <> *ALL'-' or
                inGMonthDay.delimiter <> *ALL'-');
               clear USR5010;
               USR50101 = i_value;
               AppMsg_sendEscMsg(AppMsg_newError('USR5010': USR5010));
            endif;

            // Adjust monthDay with respect to the local time zone.
            select;
            when (inGMonthDay.timeZone.ID = 'Z');    // UTC time
               gMonthDay = adjustGMonthDay(inGMonthDay);
            when (inGMonthDay.timeZone.ID = '-');    // negative time offset
               gMonthDay = adjustGMonthDay(inGMonthDay);
            when (inGMonthDay.timeZone.ID = '+');    // negative time offset
               gMonthDay = adjustGMonthDay(inGMonthDay);
            other;
               month = %dec(inGMonthDay.month: 2: 0);
               day   = %dec(inGMonthDay.day  : 2: 0);
               gMonthDay = (month * 100) + day;
            endsl;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return gMonthDay;

      /END-FREE
      *
     P UnMarshaller_togMonthDay...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'gYear'.
      *=====================================================================*
      *  Sample XML date values:   2010
      *                            2010Z              UTC time zone
      *                            2010-06:00         offset from UTC time
      *                            2010+06:00         offset from UTC time
      *=====================================================================*
     P UnMarshaller_togYear...
     P                 B                   export
      *
     D UnMarshaller_togYear...
     D                 PI                         like(wsdl_gYear_t      )
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D gYear           S                   like(wsdl_gYear_t      ) inz
      *
      *  Helper fields
     D isNegative      S               N   inz(cFalse)
     D inGYear         DS                  likeds(xmlGYear_t  ) inz
      *
      *  Messages
      /COPY QWSDL2RPG,USR5014                      Invalid XML gYear value: '&1'
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            clear inGYear;

            if (f_startsWith('-': i_value) and %len(i_value) > 1);
               inGYear = %subst(i_value: 2);
               isNegative = cTrue;
            else;
               inGYear = i_value;
               isNegative = cFalse;
            endif;

            if (not isValidTimezone(inGYear.timezone));
               clear USR5014;
               USR50141 = i_value;
               AppMsg_sendEscMsg(AppMsg_newError('USR5014': USR5014));
            endif;

            // Adjust year with respect to the local time zone.
            select;
            when (inGYear.timeZone.ID = 'Z');    // UTC time
               gYear = adjustGYear(inGYear);
            when (inGYear.timeZone.ID = '-');    // negative time offset
               gYear = adjustGYear(inGYear);
            when (inGYear.timeZone.ID = '+');    // negative time offset
               gYear = adjustGYear(inGYear);
            other;
               gYear = %dec(inGYear.year: cGYEAR_LEN: cGYEAR_DECPOS);
            endsl;

            if (isNegative);
               gYear = gYear * -1;
            endif;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return gYear;

      /END-FREE
      *
     P UnMarshaller_togYear...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML as 'gYearMonth'.
      *=====================================================================*
      *  Sample XML date values:   2010-06
      *                            2010-06Z           UTC time zone
      *                            2010-06-06:00      offset from UTC time
      *                            2010-06+06:00      offset from UTC time
      *=====================================================================*
     P UnMarshaller_togYearMonth...
     P                 B                   export
      *
     D UnMarshaller_togYearMonth...
     D                 PI                         like(wsdl_gYearMonth_t )
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D gYearMonth      S                   like(wsdl_gYearMonth_t ) inz
      *
      *  Helper fields
     D isNegative      S               N   inz(cFalse)
     D inGYearMonth    DS                  likeds(xmlGYearMonth_t) inz
     D year            S              5I 0 inz
     D month           S              5I 0 inz
      *
      *  Messages
      /COPY QWSDL2RPG,USR5011                     Invalid XML gYearMonth value: '&1'
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            clear inGYearMonth;
            if (f_startsWith('-': i_value) and %len(i_value) > 1);
               inGYearMonth = %subst(i_value: 2);
               isNegative = cTrue;
            else;
               inGYearMonth = i_value;
               isNegative = cFalse;
            endif;

            if (inGYearMonth.delimiter <> '-');
               clear USR5011;
               USR50111 = i_value;
               AppMsg_sendEscMsg(AppMsg_newError('USR5011': USR5011));
            endif;

            // Adjust yearMonth with respect to the local time zone.
            select;
            when (inGYearMonth.timeZone.ID = 'Z');    // UTC time
               gYearMonth = adjustGYearMonth(inGYearMonth);
            when (inGYearMonth.timeZone.ID = '-');    // negative time offset
               gYearMonth = adjustGYearMonth(inGYearMonth);
            when (inGYearMOnth.timeZone.ID = '+');    // negative time offset
               gYearMonth = adjustGYearMonth(inGYearMonth);
            other;
               year  = %dec(inGYearMonth.year : 4: 0);
               month = %dec(inGYearMonth.month: 2: 0);
               gYearMonth = (year * 100) + month;
            endsl;

            if (isNegative);
               gYearMonth = gYearMonth * -1;
            endif;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return gYearMonth;

      /END-FREE
      *
     P UnMarshaller_togYearMonth...
     P                 E
      *
      *=====================================================================*
      *  Decodes the Base64 encoded data of a given character field
      *  and returns the decoded data to the caller.
      *=====================================================================*
     P UnMarshaller_toBase64Binary...
     P                 B                   export
      *
     D UnMarshaller_toBase64Binary...
     D                 PI         65535A          varying
     D  i_value                   65535A   const  varying  options(*varsize)
     D  i_ccsid                      10I 0 const  options(*nopass: *omit)
      *
      *  Return value
     D                 DS
     D value                      65535A   varying inz
     D outLength               1      2U 0
      *
      *  Parameter positions
     D p_ccsid         C                   2
      *
      *  Helper fields
     D length          S             10U 0 inz
     D inValue         S                   like(i_value) inz
     D hTranscoder     S                   like(wsdl_hTranscoder_t ) inz
      *
      /COPY QWSDL2RPG,USR5008                      Length of input value exceeds maximum ...
      /COPY QWSDL2RPG,USR5009                      Length of decoded base64 value exceeds ...
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (%len(i_value) >= 65535);
               clear USR5008;
               USR50081 = 65534;
               AppMsg_sendEscMsg(AppMsg_newError('USR5008': USR5008));
            endif;

            inValue = i_value;
            length = base64_decode(%addr(inValue)+2: %len(inValue)
                                   : %addr(value)+2: 65535);

            if (length >= 65535);
               clear USR5009;
               USR50091 = 65534;
               AppMsg_sendEscMsg(AppMsg_newError('USR5009': USR5009));
            endif;

            outLength = length;

            if (%parms() >= p_ccsid and %addr(i_ccsid) <> *NULL);
               hTranscoder = Transcoder_new(0: i_ccsid);
               value = Transcoder_xlateString(hTranscoder: value);
               Transcoder_delete(hTranscoder);
            endif;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return value;

      /END-FREE
      *
     P UnMarshaller_toBase64Binary...
     P                 E
      *
      *=====================================================================*
      *  Decodes the Base64 encoded data of a given character field
      *  and returns the decoded data to the caller.
      *=====================================================================*
     P UnMarshaller_toBase64BinaryV6...
     P                 B                   export
      *
     D UnMarshaller_toBase64BinaryV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_size                       10I 0 const
     D  i_pValue                       *   const
     D  i_length                     10I 0 const
     D  i_ccsid                      10I 0 const  options(*nopass: *omit)
      *
      *  Return value
      *
      *  Parameter positions
     D p_ccsid         C                   6
      *
      *  Helper fields
     D toField         DS                  likeds(wsdl_fieldv6_t ) inz
     D maxLength       S             10I 0 inz
     D length          S             10I 0 inz
      *
      *  Local fields
     D hTranscoder     S                   like(wsdl_hTranscoder_t ) inz
     D msg             DS                  likeds(msg_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         toField.pField = i_pField;
         toField.pData  = i_pData;
         toField.size   = i_size;

         monitor;
            // Check 'varying' property
            if (not isVaryingV6(toField: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            maxLength = getMaxLengthV6(toField);

            // Check maximum length of data of target field
            if (isTooShortV6(toField: i_length: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            length = base64_decode(i_pValue: i_length
                                   : toField.pData: maxLength);

            // Check maximum length of data of target field
            if (isTooShortV6(toField: length: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            if (%parms() >= p_ccsid and %addr(i_ccsid) <> *NULL);
               hTranscoder = Transcoder_new(0: i_ccsid);
               length = Transcoder_xlateBuffer(
                              hTranscoder: toField.pData: length: maxLength);
               Transcoder_delete(hTranscoder);

               // Check maximum length of data of target field
               if (isTooShortV6(toField: length: msg));
                  AppMsg_sendEscMsg(msg);
               endif;
            endif;

            // Set length of copied data
            setLengthV6(toField: length);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessageV6(i_pValue: i_length));
         endmon;

      /END-FREE
      *
     P UnMarshaller_toBase64BinaryV6...
     P                 E
      *
      *=====================================================================*
      *  Decodes the Base64 encoded data of a given buffer and
      *  writes the decoded data to the specified IFS path.
      *=====================================================================*
     P UnMarshaller_fromBase64BufferToStmf...
     P                 B                   export
      *
     D UnMarshaller_fromBase64BufferToStmf...
     D                 PI            10I 0
     D  i_path                     5000A   const  varying  options(*varsize)
     D  i_pBuffer                      *   const
     D  i_size                       10I 0 const
     D  i_codepage                   10I 0 const  options(*nopass: *omit)
      *
      *  Return value
     D size            S             10I 0 inz(-1)
      *
      *  Parameter positions
     D p_codepage      C                   4
      *
      *  Optional parameter fields
     D codepage        S                   like(i_codepage) inz
      *
      *  Helper fields
     D fd              S             10I 0 inz
      *
      *  Messages
      /COPY QWSDL2RPG,USR5015                      Could not open stream file &1.
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (%parms() >= p_codepage and %addr(i_codepage) <> *NULL);
               codepage = i_codepage;
            else;
               codepage = 819;
            endif;

            f_clrErrno();

            fd = open(File_fixPath(i_path): O_SHARE_RDONLY + O_WRONLY +
                                            O_CREAT + O_EXCL +
                                            O_CODEPAGE
                                          : S_IRWXU + S_IRWXG + S_IRWXO
                                          : codepage);
            if (fd = -1);
               clear USR5015;
               USR50151 = i_path;
               USR50152 = f_errno();
               USR50153 = f_strerror(f_errno());
               AppMsg_sendEscMsg(AppMsg_newError('USR5015': USR5015));
            endif;

            size = base64_decodeBufferToStream(
                      i_pBuffer: i_size: fd: %paddr('write'));

            callp close(fd);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessageV6(i_pBuffer: i_size));
         endmon;

         return size;

      /END-FREE
      *
     P UnMarshaller_fromBase64BufferToStmf...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML 'language' value as RPG string.
      *=====================================================================*
     P UnMarshaller_toLanguage...
     P                 B                   export
      *
     D UnMarshaller_toLanguage...
     D                 PI         65535A          varying
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (%len(i_value) > 0);
               return %subst(i_value: 1: %len(i_value));
            else;
               return '';
            endif;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

      /END-FREE
      *
     P UnMarshaller_toLanguage...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML 'language' value as RPG string.
      *=====================================================================*
     P UnMarshaller_toLanguageV6...
     P                 B                   export
      *
     D UnMarshaller_toLanguageV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_size                       10I 0 const
     D  i_pValue                       *   const
     D  i_length                     10I 0 const
      *
      *  Return value
      *
      *  Helper fields
     D toField         DS                  likeds(wsdl_fieldv6_t ) inz
      *
      *  Local fields
     D msg             DS                  likeds(msg_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         toField.pField = i_pField;
         toField.pData  = i_pData;
         toField.size   = i_size;

         monitor;
            // Check 'varying' property
            if (not isVaryingV6(toField: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Check maximum length of data of target field
            if (isTooShortV6(toField: i_length: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Copy data to field
            if (not copyValueV6(toField: i_pValue: i_length: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Set length of copied data
            setLengthV6(toField: i_length);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessageV6(i_pValue: i_length));
         endmon;

      /END-FREE
      *
     P UnMarshaller_toLanguageV6...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML 'token' value as RPG string.
      *=====================================================================*
     P UnMarshaller_toToken...
     P                 B                   export
      *
     D UnMarshaller_toToken...
     D                 PI         65535A          varying
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (%len(i_value) > 0);
               return %subst(i_value: 1: %len(i_value));
            else;
               return '';
            endif;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

      /END-FREE
      *
     P UnMarshaller_toToken...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML 'token' value as RPG string.
      *=====================================================================*
     P UnMarshaller_toTokenV6...
     P                 B                   export
      *
     D UnMarshaller_toTokenV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_size                       10I 0 const
     D  i_pValue                       *   const
     D  i_length                     10I 0 const
      *
      *  Return value
      *
      *  Helper fields
     D toField         DS                  likeds(wsdl_fieldv6_t ) inz
      *
      *  Local fields
     D msg             DS                  likeds(msg_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         toField.pField = i_pField;
         toField.pData  = i_pData;
         toField.size   = i_size;

         monitor;
            // Check 'varying' property
            if (not isVaryingV6(toField: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Check maximum length of data of target field
            if (isTooShortV6(toField: i_length: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Copy data to field
            if (not copyValueV6(toField: i_pValue: i_length: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Set length of copied data
            setLengthV6(toField: i_length);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessageV6(i_pValue: i_length));
         endmon;

      /END-FREE
      *
     P UnMarshaller_toTokenV6...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML 'anyURI' value as RPG string.
      *=====================================================================*
     P UnMarshaller_toAnyURI...
     P                 B                   export
      *
     D UnMarshaller_toAnyURI...
     D                 PI         65535A          varying
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (%len(i_value) > 0);
               return %subst(i_value: 1: %len(i_value));
            else;
               return '';
            endif;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

      /END-FREE
      *
     P UnMarshaller_toAnyURI...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML 'anyURI' value as RPG string.
      *=====================================================================*
     P UnMarshaller_toAnyURIV6...
     P                 B                   export
      *
     D UnMarshaller_toAnyURIV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_size                       10I 0 const
     D  i_pValue                       *   const
     D  i_length                     10I 0 const
      *
      *  Return value
      *
      *  Helper fields
     D toField         DS                  likeds(wsdl_fieldv6_t ) inz
      *
      *  Local fields
     D msg             DS                  likeds(msg_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         toField.pField = i_pField;
         toField.pData  = i_pData;
         toField.size   = i_size;

         monitor;
            // Check 'varying' property
            if (not isVaryingV6(toField: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Check maximum length of data of target field
            if (isTooShortV6(toField: i_length: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Copy data to field
            if (not copyValueV6(toField: i_pValue: i_length: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Set length of copied data
            setLengthV6(toField: i_length);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessageV6(i_pValue: i_length));
         endmon;

      /END-FREE
      *
     P UnMarshaller_toAnyURIV6...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML 'AttributedURI' value as RPG string.
      *=====================================================================*
     P UnMarshaller_toAttributedURI...
     P                 B                   export
      *
     D UnMarshaller_toAttributedURI...
     D                 PI         65535A          varying
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (%len(i_value) > 0);
               return %subst(i_value: 1: %len(i_value));
            else;
               return '';
            endif;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

      /END-FREE
      *
     P UnMarshaller_toAttributedURI...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML 'AttributedURI' value as RPG string.
      *=====================================================================*
     P UnMarshaller_toAttributedURIV6...
     P                 B                   export
      *
     D UnMarshaller_toAttributedURIV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_size                       10I 0 const
     D  i_pValue                       *   const
     D  i_length                     10I 0 const
      *
      *  Return value
      *
      *  Helper fields
     D toField         DS                  likeds(wsdl_fieldv6_t ) inz
      *
      *  Local fields
     D msg             DS                  likeds(msg_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         toField.pField = i_pField;
         toField.pData  = i_pData;
         toField.size   = i_size;

         monitor;
            // Check 'varying' property
            if (not isVaryingV6(toField: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Check maximum length of data of target field
            if (isTooShortV6(toField: i_length: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Copy data to field
            if (not copyValueV6(toField: i_pValue: i_length: msg));
               AppMsg_sendEscMsg(msg);
            endif;

            // Set length of copied data
            setLengthV6(toField: i_length);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessageV6(i_pValue: i_length));
         endmon;

      /END-FREE
      *
     P UnMarshaller_toAttributedURIV6...
     P                 E
      *
      *=====================================================================*
      *  Returns a given XML 'duration' value as RPG structure.
      *=====================================================================*
     P UnMarshaller_toDuration...
     P                 B                   export
      *
     D UnMarshaller_toDuration...
     D                 PI                         likeds(wsdl_duration_t )
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D duration        DS                  likeds(wsdl_duration_t ) inz
      *
      *  Helper fields
     D s               S             10I 0 inz
     D p               S             10I 0 inz
     D x               S             10I 0 inz
     D sNum            S            128A   varying inz
     D char            S              1A   inz
     D pActive         S              4A   varying inz
     D pDate           C                   'YMDT'
     D pTime           C                   'HMS'
     D isDate          S               N   inz
     D isPatternError  S               N   inz(cFalse)
      *
      *  Messages
      /COPY QWSDL2RPG,USR5004
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (%len(i_value) = 0);
               clear USR5004;
               USR50041 = i_value;
               AppMsg_sendEscMsg(AppMsg_newError('USR5004': USR5004));
            endif;

            if (%subst(i_value: 1: 1) <> 'P' and %subst(i_value: 1: 1) <> '-');
               clear USR5004;
               USR50041 = i_value;
               AppMsg_sendEscMsg(AppMsg_newError('USR5004': USR5004));
            endif;

            if (%subst(i_value: 1: 1) = '-');
               if (%len(i_value) = 1 or %subst(i_value: 2: 1) <> 'P');
                  clear USR5004;
                  USR50041 = i_value;
                  AppMsg_sendEscMsg(AppMsg_newError('USR5004': USR5004));
               endif;
               duration.isNegative = cTrue;
               s = 3;
            else;
               duration.isNegative = cFalse;
               s = 2;
            endif;

            isDate  = cTrue;
            pActive = pDate;

            for p = s to %len(i_value);

               char = %subst(i_value: p: 1);

               select;
               when (isDate and char = 'Y');
                  duration.years = f_val(sNum);
                  x = %scan(char: pActive);
                  if (x = 0);
                     isPatternError = cTrue;
                     leave;
                  endif;

               when (isDate and char = 'M');
                  duration.months = f_val(sNum);
                  x = %scan(char: pActive);
                  if (x = 0);
                     isPatternError = cTrue;
                     leave;
                  endif;

               when (isDate and char = 'D');
                  duration.days = f_val(sNum);
                  x = %scan(char: pActive);
                  if (x = 0);
                     isPatternError = cTrue;
                     leave;
                  endif;

               when (isDate and char = 'T');
                  isDate  = cFalse;
                  pActive = pTime;
                  x = 0;

               when (not isDate and char = 'H');
                  duration.hours = f_val(sNum);
                  x = %scan(char: pActive);
                  if (x = 0);
                     isPatternError = cTrue;
                     leave;
                  endif;

               when (not isDate and char = 'M');
                  duration.minutes = f_val(sNum);
                  x = %scan(char: pActive);
                  if (x = 0);
                     isPatternError = cTrue;
                     leave;
                  endif;

               when (not isDate and char = 'S');
                  duration.seconds = f_val(sNum);
                  x = %scan(char: pActive);
                  if (x = 0);
                     isPatternError = cTrue;
                     leave;
                  endif;

               other;
                  sNum = sNum + char;
                  x = 0;
               endsl;

               if (x > 0);
                  %subst(pActive: 1: x) = f_space(x);
                  sNum = '';
               endif;

            endfor;

            if (isPatternError);
               clear USR5004;
               USR50041 = i_value;
               AppMsg_sendEscMsg(AppMsg_newError('USR5004': USR5004));
            endif;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return duration;

      /END-FREE
      *
     P UnMarshaller_toDuration...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG anyType value.
      *=====================================================================*
     P Marshaller_toAnyType...
     P                 B                   export
      *
     D Marshaller_toAnyType...
     D                 PI         65535A          varying
     D  i_value                   65535A          varying  options(*varsize)
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Helper fields
     D fromValue       DS                  likeds(wsdl_fieldv6_t )
     D toValue         DS                  likeds(wsdl_fieldv6_t )
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            fromValue.pField = %addr(i_value);
            fromValue.pData  = %addr(i_value) + 2;
            fromValue.size   = %len(i_value);

            toValue.pField = %addr(value);
            toValue.pData  = %addr(value) + 2;
            toValue.size   = %size(value) - 2;

            encodeXmlEntityToFieldV6(fromValue: toValue);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toAnyType...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG anyType value.
      *=====================================================================*
     P Marshaller_toAnyTypeV6...
     P                 B                   export
      *
     D Marshaller_toAnyTypeV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_length                     10I 0 const
     D  i_hOutStream                       const  like(wsdl_hStream_t)
     D  i_pWriteProc                   *   const  procptr
     D                                            options(*nopass: *omit)
      *
      *  Parameter positions
     D p_pWriteProc    C                   5
      *
      *  Optional parameter fields
     D pWriteProc      S                   like(i_pWriteProc) inz
      *
      *  Helper fields
     D fromValue       DS                  likeds(wsdl_fieldv6_t )
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (%parms() >= p_pWriteProc and %addr(i_pWriteProc) <> *NULL);
               pWriteProc = i_pWriteProc;
            else;
               pWriteProc = getDefaultWriteProc();
            endif;

            fromValue.pField = i_pField;
            fromValue.pData  = i_pData;
            fromValue.size   = i_length;

            encodeXmlEntityToStream(
                  fromValue: i_hOutStream: pWriteProc);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessageV6(i_pData: i_length));
         endmon;

         return;

      /END-FREE
      *
     P Marshaller_toAnyTypeV6...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG anySimpleType value.
      *=====================================================================*
     P Marshaller_toAnySimpleType...
     P                 B                   export
      *
     D Marshaller_toAnySimpleType...
     D                 PI         65535A          varying
     D  i_value                   65535A          varying  options(*varsize)
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Helper fields
     D fromValue       DS                  likeds(wsdl_fieldv6_t )
     D toValue         DS                  likeds(wsdl_fieldv6_t )
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            fromValue.pField = %addr(i_value);
            fromValue.pData  = %addr(i_value) + 2;
            fromValue.size   = %len(i_value);

            toValue.pField = %addr(value);
            toValue.pData  = %addr(value) + 2;
            toValue.size   = %size(value) - 2;

            encodeXmlEntityToFieldV6(fromValue: toValue);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toAnySimpleType...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG anySimpleType value.
      *=====================================================================*
     P Marshaller_toAnySimpleTypeV6...
     P                 B                   export
      *
     D Marshaller_toAnySimpleTypeV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_length                     10I 0 const
     D  i_hOutStream                       const  like(wsdl_hStream_t)
     D  i_pWriteProc                   *   const  procptr
     D                                            options(*nopass: *omit)
      *
      *  Parameter positions
     D p_pWriteProc    C                   5
      *
      *  Optional parameter fields
     D pWriteProc      S                   like(i_pWriteProc) inz
      *
      *  Helper fields
     D fromValue       DS                  likeds(wsdl_fieldv6_t )
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (%parms() >= p_pWriteProc and %addr(i_pWriteProc) <> *NULL);
               pWriteProc = i_pWriteProc;
            else;
               pWriteProc = getDefaultWriteProc();
            endif;

            fromValue.pField = i_pField;
            fromValue.pData  = i_pData;
            fromValue.size   = i_length;

            encodeXmlEntityToStream(
                  fromValue: i_hOutStream: pWriteProc);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessageV6(i_pData: i_length));
         endmon;

         return;

      /END-FREE
      *
     P Marshaller_toAnySimpleTypeV6...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG 'any' value.
      *=====================================================================*
     P Marshaller_toAny...
     P                 B                   export
      *
     D Marshaller_toAny...
     D                 PI         65535A          varying
     D  i_value                   65535A          varying  options(*varsize)
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Helper fields
     D fromValue       DS                  likeds(wsdl_fieldv6_t )
     D toValue         DS                  likeds(wsdl_fieldv6_t )
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            fromValue.pField = %addr(i_value);
            fromValue.pData  = %addr(i_value) + 2;
            fromValue.size   = %len(i_value);

            toValue.pField = %addr(value);
            toValue.pData  = %addr(value) + 2;
            toValue.size   = %size(value) - 2;

            encodeXmlEntityToFieldV6(fromValue: toValue);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toAny...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG 'any' value.
      *=====================================================================*
     P Marshaller_toAnyV6...
     P                 B                   export
      *
     D Marshaller_toAnyV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_length                     10I 0 const
     D  i_hOutStream                       const  like(wsdl_hStream_t)
     D  i_pWriteProc                   *   const  procptr
     D                                            options(*nopass: *omit)
      *
      *  Parameter positions
     D p_pWriteProc    C                   5
      *
      *  Optional parameter fields
     D pWriteProc      S                   like(i_pWriteProc) inz
      *
      *  Helper fields
     D fromValue       DS                  likeds(wsdl_fieldv6_t )
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (%parms() >= p_pWriteProc and %addr(i_pWriteProc) <> *NULL);
               pWriteProc = i_pWriteProc;
            else;
               pWriteProc = getDefaultWriteProc();
            endif;

            fromValue.pField = i_pField;
            fromValue.pData  = i_pData;
            fromValue.size   = i_length;

            encodeXmlEntityToStream(
                  fromValue: i_hOutStream: pWriteProc);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessageV6(i_pData: i_length));
         endmon;

         return;

      /END-FREE
      *
     P Marshaller_toAnyV6...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG string value.
      *=====================================================================*
     P Marshaller_toString...
     P                 B                   export
      *
     D Marshaller_toString...
     D                 PI         65535A          varying
     D  i_value                   65535A          varying  options(*varsize)
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Helper fields
     D fromValue       DS                  likeds(wsdl_fieldv6_t )
     D toValue         DS                  likeds(wsdl_fieldv6_t )
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            fromValue.pField = %addr(i_value);
            fromValue.pData  = %addr(i_value) + 2;
            fromValue.size   = %len(i_value);

            toValue.pField = %addr(value);
            toValue.pData  = %addr(value) + 2;
            toValue.size   = %size(value) - 2;

            encodeXmlEntityToFieldV6(fromValue: toValue);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toString...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG string value.
      *=====================================================================*
     P Marshaller_toStringV6...
     P                 B                   export
      *
     D Marshaller_toStringV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_length                     10I 0 const
     D  i_hOutStream                       const  like(wsdl_hStream_t)
     D  i_pWriteProc                   *   const  procptr
     D                                            options(*nopass: *omit)
      *
      *  Parameter positions
     D p_pWriteProc    C                   5
      *
      *  Optional parameter fields
     D pWriteProc      S                   like(i_pWriteProc) inz
      *
      *  Helper fields
     D fromValue       DS                  likeds(wsdl_fieldv6_t )
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (%parms() >= p_pWriteProc and %addr(i_pWriteProc) <> *NULL);
               pWriteProc = i_pWriteProc;
            else;
               pWriteProc = getDefaultWriteProc();
            endif;

            fromValue.pField = i_pField;
            fromValue.pData  = i_pData;
            fromValue.size   = i_length;

            encodeXmlEntityToStream(
                  fromValue: i_hOutStream: pWriteProc);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessageV6(i_pData: i_length));
         endmon;

         return;

      /END-FREE
      *
     P Marshaller_toStringV6...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG normalizedString value.
      *=====================================================================*
     P Marshaller_toNormalizedString...
     P                 B                   export
      *
     D Marshaller_toNormalizedString...
     D                 PI         65535A          varying
     D  i_value                   65535A          varying  options(*varsize)
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Helper fields
     D x               S             10I 0 inz
     D char            S              1A   inz
     D fromValue       DS                  likeds(wsdl_fieldv6_t )
     D toValue         DS                  likeds(wsdl_fieldv6_t )
      *
      *  Messages
      /COPY QWSDL2RPG,USR5016                     Invalid normalized string: '&1'
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            for x = 1 to %len(i_value);
               char = %subst(i_value: x: 1);
               if (not isNormalizedStringChar(char));
                  clear USR5016;
                  USR50161 = i_value;
                  USR50162 = f_this();
                  USR50163 = x;
                  AppMsg_sendEscMsg(AppMsg_new('USR5016': USR5016));
               endif;
            endfor;

            fromValue.pField = %addr(i_value);
            fromValue.pData  = %addr(i_value) + 2;
            fromValue.size   = %len(i_value);

            toValue.pField = %addr(value);
            toValue.pData  = %addr(value) + 2;
            toValue.size   = %size(value) - 2;

            encodeXmlEntityToFieldV6(fromValue: toValue);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toNormalizedString...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG normalizedString value.
      *=====================================================================*
     P Marshaller_toNormalizedStringV6...
     P                 B                   export
      *
     D Marshaller_toNormalizedStringV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_length                     10I 0 const
     D  i_hOutStream                       const  like(wsdl_hStream_t)
     D  i_pWriteProc                   *   const  procptr
     D                                            options(*nopass: *omit)
      *
      *  Parameter positions
     D p_pWriteProc    C                   5
      *
      *  Optional parameter fields
     D pWriteProc      S                   like(i_pWriteProc) inz
      *
      *  Helper fields
     D x               S             10I 0 inz
     D char            S              1A   based(pChar)
     D data            S            128A   based(i_pData)
     D value           S            128A   varying inz
     D fromValue       DS                  likeds(wsdl_fieldv6_t )
      *
      *  Messages
      /COPY QWSDL2RPG,USR5016                     Invalid normalized string: '&1'
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            for x = 1 to i_length;
               pChar = i_pData + (x-1);
               if (not isNormalizedStringChar(char));
                  if (i_length >= %len(data));
                     value = %subst(data: 1: %len(data));
                  else;
                     value = %subst(data: 1: i_length);
                  endif;
                  clear USR5016;
                  USR50161 = value;
                  USR50162 = f_this();
                  USR50163 = x;
                  AppMsg_sendEscMsg(AppMsg_new('USR5016': USR5016));
               endif;
            endfor;

            if (%parms() >= p_pWriteProc and %addr(i_pWriteProc) <> *NULL);
               pWriteProc = i_pWriteProc;
            else;
               pWriteProc = getDefaultWriteProc();
            endif;

            fromValue.pField = i_pField;
            fromValue.pData  = i_pData;
            fromValue.size   = i_length;

            encodeXmlEntityToStream(
                  fromValue: i_hOutStream: pWriteProc);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessageV6(i_pData: i_length));
         endmon;

         return;

      /END-FREE
      *
     P Marshaller_toNormalizedStringV6...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG QName value.
      *=====================================================================*
     P Marshaller_toQName...
     P                 B                   export
      *
     D Marshaller_toQName...
     D                 PI         65535A          varying
     D  i_value                   65535A          varying  options(*varsize)
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Helper fields
     D fromValue       DS                  likeds(wsdl_fieldv6_t )
     D toValue         DS                  likeds(wsdl_fieldv6_t )
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            fromValue.pField = %addr(i_value);
            fromValue.pData  = %addr(i_value) + 2;
            fromValue.size   = %len(i_value);

            toValue.pField = %addr(value);
            toValue.pData  = %addr(value) + 2;
            toValue.size   = %size(value) - 2;

            encodeXmlEntityToFieldV6(fromValue: toValue);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toQName...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG QName value.
      *=====================================================================*
     P Marshaller_toQNameV6...
     P                 B                   export
      *
     D Marshaller_toQNameV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_length                     10I 0 const
     D  i_hOutStream                       const  like(wsdl_hStream_t)
     D  i_pWriteProc                   *   const  procptr
     D                                            options(*nopass: *omit)
      *
      *  Parameter positions
     D p_pWriteProc    C                   5
      *
      *  Optional parameter fields
     D pWriteProc      S                   like(i_pWriteProc) inz
      *
      *  Helper fields
     D data            S            128A   based(i_pData)
     D value           S            128A   varying inz
     D fromValue       DS                  likeds(wsdl_fieldv6_t )
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (%parms() >= p_pWriteProc and %addr(i_pWriteProc) <> *NULL);
               pWriteProc = i_pWriteProc;
            else;
               pWriteProc = getDefaultWriteProc();
            endif;

            fromValue.pField = i_pField;
            fromValue.pData  = i_pData;
            fromValue.size   = i_length;

            encodeXmlEntityToStream(
                  fromValue: i_hOutStream: pWriteProc);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessageV6(i_pData: i_length));
         endmon;

         return;

      /END-FREE
      *
     P Marshaller_toQNameV6...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG boolean value.
      *=====================================================================*
     P Marshaller_toBoolean...
     P                 B                   export
      *
     D Marshaller_toBoolean...
     D                 PI         65535A          varying
     D  i_boolean                      N   const
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Messages
      /COPY QWSDL2RPG,USR5001                     Failed to convert boolean value '&1' to XML.
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            select;
            when (i_boolean = cTrue);
               value = 'true';
            when (i_boolean = cFalse);
               value = 'false';
            other;
               clear USR5001;
               USR50011 = i_boolean;
               AppMsg_sendEscMsgAndDump(AppMsg_new('USR5001': USR5001));
            endsl;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_boolean));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toBoolean...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG decimal value.
      *=====================================================================*
     P Marshaller_toDecimal...
     P                 B                   export
      *
     D Marshaller_toDecimal...
     D                 PI         65535A          varying
     D  i_decimal                          const  like(wsdl_decimal_t )
      *
      *  Return value
     D value           S          65535A   varying inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            value = editDecimal(i_decimal);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toDecimal...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG float value.
      *=====================================================================*
     P Marshaller_toFloat...
     P                 B                   export
      *
     D Marshaller_toFloat...
     D                 PI         65535A          varying
     D  i_float                            const  like(wsdl_float_t   )
      *
      *  Return value
     D value           S          65535A   varying inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            value = %char(i_float);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toFloat...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG double value.
      *=====================================================================*
     P Marshaller_toDouble...
     P                 B                   export
      *
     D Marshaller_toDouble...
     D                 PI         65535A          varying
     D  i_double                           const  like(wsdl_double_t  )
      *
      *  Return value
     D value           S          65535A   varying inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            value = %char(i_double);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toDouble...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG short value.
      *=====================================================================*
     P Marshaller_toShort...
     P                 B                   export
      *
     D Marshaller_toShort...
     D                 PI         65535A          varying
     D  i_short                            const  like(wsdl_short_t )
      *
      *  Return value
     D value           S          65535A   varying inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            value = %char(i_short);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toShort...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG byte value.
      *=====================================================================*
     P Marshaller_toByte...
     P                 B                   export
      *
     D Marshaller_toByte...
     D                 PI         65535A          varying
     D  i_byte                             const  like(wsdl_byte_t  )
      *
      *  Return value
     D value           S          65535A   varying inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            value = %char(i_byte);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toByte...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG integer value.
      *=====================================================================*
     P Marshaller_toInt...
     P                 B                   export
      *
     D Marshaller_toInt...
     D                 PI         65535A          varying
     D  i_int                              const  like(wsdl_int_t     )
      *
      *  Return value
     D value           S          65535A   varying inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            value = %char(i_int);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toInt...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG integer value.
      *=====================================================================*
     P Marshaller_toInteger...
     P                 B                   export
      *
     D Marshaller_toInteger...
     D                 PI         65535A          varying
     D  i_integer                          const  like(wsdl_integer_t )
      *
      *  Return value
     D value           S          65535A   varying inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            value = %char(i_integer);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toInteger...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG long integer value.
      *=====================================================================*
     P Marshaller_toLong...
     P                 B                   export
      *
     D Marshaller_toLong...
     D                 PI         65535A          varying
     D  i_long                             const  like(wsdl_long_t    )
      *
      *  Return value
     D value           S          65535A   varying inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            value = %char(i_long);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toLong...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG unsigned short value.
      *=====================================================================*
     P Marshaller_toUnsignedShort...
     P                 B                   export
      *
     D Marshaller_toUnsignedShort...
     D                 PI         65535A          varying
     D  i_uShort                           const  like(wsdl_uShort_t)
      *
      *  Return value
     D value           S          65535A   varying inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            value = %char(i_uShort);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toUnsignedShort...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG unsigned byte value.
      *=====================================================================*
     P Marshaller_toUnsignedByte...
     P                 B                   export
      *
     D Marshaller_toUnsignedByte...
     D                 PI         65535A          varying
     D  i_uByte                            const  like(wsdl_uByte_t )
      *
      *  Return value
     D value           S          65535A   varying inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            value = %char(i_uByte);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toUnsignedByte...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG unsigned integer value.
      *=====================================================================*
     P Marshaller_toUnsignedInt...
     P                 B                   export
      *
     D Marshaller_toUnsignedInt...
     D                 PI         65535A          varying
     D  i_uInt                             const  like(wsdl_uInt_t    )
      *
      *  Return value
     D value           S          65535A   varying inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            value = %char(i_uInt);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toUnsignedInt...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG unsigned long integer value.
      *=====================================================================*
     P Marshaller_toUnsignedLong...
     P                 B                   export
      *
     D Marshaller_toUnsignedLong...
     D                 PI         65535A          varying
     D  i_uLong                            const  like(wsdl_uLong_t   )
      *
      *  Return value
     D value           S          65535A   varying inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;
            value = %char(i_uLong);
            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toUnsignedLong...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG non negative integer value.
      *=====================================================================*
     P Marshaller_toNonNegativeInteger...
     P                 B                   export
      *
     D Marshaller_toNonNegativeInteger...
     D                 PI         65535A          varying
     D  i_nonNegInt                        const  like(wsdl_nonNegInt_t )
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Messages
      /COPY QWSDL2RPG,USR5003
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (i_nonNegInt < 0);
               clear USR5003;
               USR50031 = %char(i_nonNegInt);
               USR50032 = 1;   // negative values are not allowed
               AppMsg_sendEscMsg(AppMsg_newError('USR5003': USR5003));
            endif;

            value = %char(i_nonNegInt);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toNonNegativeInteger...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG non positive integer value.
      *=====================================================================*
     P Marshaller_toNonPositiveInteger...
     P                 B                   export
      *
     D Marshaller_toNonPositiveInteger...
     D                 PI         65535A          varying
     D  i_nonPosInt                        const  like(wsdl_nonPosInt_t )
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Messages
      /COPY QWSDL2RPG,USR5003
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (i_nonPosInt > 0);
               clear USR5003;
               USR50031 = %char(i_nonPosInt);
               USR50032 = 2;   // positive values are not allowed
               AppMsg_sendEscMsg(AppMsg_newError('USR5003': USR5003));
            endif;

            value = %char(i_nonPosInt);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toNonPositiveInteger...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG negative integer value.
      *=====================================================================*
     P Marshaller_toNegativeInteger...
     P                 B                   export
      *
     D Marshaller_toNegativeInteger...
     D                 PI         65535A          varying
     D  i_negInt                           const  like(wsdl_negInt_t )
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Messages
      /COPY QWSDL2RPG,USR5003
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (i_negInt >= 0);
               clear USR5003;
               USR50031 = %char(i_negInt);
               USR50032 = 3;   // positive values and 0 are not allowed
               AppMsg_sendEscMsg(AppMsg_newError('USR5003': USR5003));
            endif;

            value = %char(i_negInt);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toNegativeInteger...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG positive integer value.
      *=====================================================================*
     P Marshaller_toPositiveInteger...
     P                 B                   export
      *
     D Marshaller_toPositiveInteger...
     D                 PI         65535A          varying
     D  i_posInt                           const  like(wsdl_posInt_t )
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Messages
      /COPY QWSDL2RPG,USR5003
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (i_posInt <= 0);
               clear USR5003;
               USR50031 = %char(i_posInt);
               USR50032 = 4;   // negative values and 0 are not allowed
               AppMsg_sendEscMsg(AppMsg_newError('USR5003': USR5003));
            endif;

            value = %char(i_posInt);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toPositiveInteger...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG timestamp value.
      *=====================================================================*
      *  Sample java.util.Date value:   2007-01-11T23:00:00.000Z
      *  Sample RPG timestamp value :   2007-01-11-23.00.00.000000
      *=====================================================================*
     P Marshaller_toDateTime...
     P                 B                   export
      *
     D Marshaller_toDateTime...
     D                 PI         65535A          varying
     D  i_timestamp                    Z   const
     D  i_timeZone                    6A   const  options(*nopass)
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Parameter positions
     D p_timeZone      C                   2
      *
      *  Helper fields
     D outTimestamp    DS                  qualified
     D  year                          4A
     D  minus_1                       1A
     D  month                         2A
     D  minus_2                       1A
     D  day                           2A
     D  char_T                        1A
     D  hour                          2A
     D  colon_1                       1A
     D  minute                        2A
     D  colon_2                       1A
     D  second                        2A
     D  dot                           1A
     D  msec                          3A
     D  timeZone                           likeds(xmlTimeZone_t )
      *
     D inTimestamp     DS                  qualified
     D  year                          4A
     D  minus_1                       1A
     D  month                         2A
     D  minus_2                       1A
     D  day                           2A
     D  minus_3                       1A
     D  hour                          2A
     D  dot_1                         1A
     D  minute                        2A
     D  dot_2                         1A
     D  second                        2A
     D  dot_3                         1A
     D  msec                          3A
     D  usec                          3A
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            inTimestamp = %char(i_timestamp);

            clear outTimestamp;
            outTimestamp.year    = inTimestamp.year;
            outTimestamp.minus_1 = inTimestamp.minus_1;
            outTimestamp.month   = inTimestamp.month;
            outTimestamp.minus_2 = inTimestamp.minus_2;
            outTimestamp.day     = inTimestamp.day;
            outTimestamp.char_T  = 'T';
            outTimestamp.hour    = inTimestamp.hour;
            outTimestamp.colon_1 = ':';
            outTimestamp.minute  = inTimestamp.minute;
            outTimestamp.colon_2 = ':';
            outTimestamp.second  = inTimestamp.second;
            outTimestamp.dot     = '.';
            outTimestamp.msec    = inTimestamp.msec;   // milliseconds
                                                       // no microseconds in XML
            if (%parms() >= p_timeZone);
               outTimestamp.timeZone = getXmlTimeZone(i_timeZone);
            else;
               outTimestamp.timeZone = getXmlTimeZone(WSDL_LOCAL_TIMEZONE);
            endif;

            value = %trimR(outTimestamp);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toDateTime...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG date value.
      *=====================================================================*
     P Marshaller_toDate...
     P                 B                   export
      *
     D Marshaller_toDate...
     D                 PI         65535A          varying
     D  i_date                         D   const
     D  i_timeZone                    6A   const  options(*nopass)
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Parameter positions
     D p_timeZone      C                   2
      *
      *  Helper fields
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            value = %char(i_date: *ISO);

            if (%parms() >= p_timeZone);
               value = value + %trimR(getXmlTimeZone(i_timeZone));
            else;
               value = value + %trimR(getXmlTimeZone(WSDL_LOCAL_TIMEZONE));
            endif;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toDate...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG time value.
      *=====================================================================*
     P Marshaller_toTime...
     P                 B                   export
      *
     D Marshaller_toTime...
     D                 PI         65535A          varying
     D  i_time                         T   const
     D  i_timeZone                    6A   const  options(*nopass)
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Parameter positions
     D p_timeZone      C                   2
      *
      *  Helper fields
     D outTime         DS                  likeds(timeValueShort_t) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            outTime = %char(i_time: *ISO);
            outTime.dot_1 = ':';
            outTime.dot_2 = ':';

            value = outTime;

            if (%parms() >= p_timeZone);
               value = value + %trimR(getXmlTimeZone(i_timeZone));
            else;
               value = value + %trimR(getXmlTimeZone(WSDL_LOCAL_TIMEZONE));
            endif;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toTime...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG gDay value.
      *=====================================================================*
     P Marshaller_togDay...
     P                 B                   export
      *
     D Marshaller_togDay...
     D                 PI         65535A          varying
     D  i_gDay                             const  like(wsdl_gDay_t       )
     D  i_timeZone                    6A   const  options(*nopass)
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Parameter positions
     D p_timeZone      C                   2
      *
      *  Helper fields
     D outGDay         DS                  likeds(xmlGDay_t       ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            outGDay.prefix = *ALL'-';
            outGDay.day    = %editc(%dec(i_gDay: 2: 0): 'X');

            if (%parms() >= p_timeZone);
               outGDay.timeZone = getXmlTimeZone(i_timeZone);
            else;
               outGDay.timeZone = getXmlTimeZone(WSDL_LOCAL_TIMEZONE);
            endif;

            value = %trimR(outGDay);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_togDay...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG gMonth value.
      *=====================================================================*
     P Marshaller_togMonth...
     P                 B                   export
      *
     D Marshaller_togMonth...
     D                 PI         65535A          varying
     D  i_gMonth                           const  like(wsdl_gMonth_t     )
     D  i_timeZone                    6A   const  options(*nopass)
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Parameter positions
     D p_timeZone      C                   2
      *
      *  Helper fields
     D outGMonth       DS                  likeds(xmlGMonth_t     ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            outGMonth.prefix    = *ALL'-';
            outGMonth.month     = %editc(%dec(i_gMonth: 2: 0): 'X');
            outGMonth.delimiter = *ALL'-';

            if (%parms() >= p_timeZone);
               outGMonth.timeZone = getXmlTimeZone(i_timeZone);
            else;
               outGMonth.timeZone = getXmlTimeZone(WSDL_LOCAL_TIMEZONE);
            endif;

            value = %trimR(outGMonth);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_togMonth...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG gMOnthDay value.
      *=====================================================================*
     P Marshaller_togMonthDay...
     P                 B                   export
      *
     D Marshaller_togMonthDay...
     D                 PI         65535A          varying
     D  i_gMonthDay                        const  like(wsdl_gMonthDay_t  )
     D  i_timeZone                    6A   const  options(*nopass)
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Parameter positions
     D p_timeZone      C                   2
      *
      *  Helper fields
     D monthDay        DS                  qualified
     D  month                         2S 0 inz
     D  day                           2S 0 inz
     D  value                  1      4S 0
      *
     D outGMonthDay    DS                  likeds(xmlGMonthDay_t  ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            monthDay.value = i_gMonthDay;

            outGMonthDay.prefix    = *ALL'-';
            outGMonthDay.month     = %editc(monthDay.month: 'X');
            outGMonthDay.delimiter = '-';
            outGMonthDay.day       = %editc(monthDay.day: 'X');

            if (%parms() >= p_timeZone);
               outGMonthDay.timeZone = getXmlTimeZone(i_timeZone);
            else;
               outGMonthDay.timeZone = getXmlTimeZone(WSDL_LOCAL_TIMEZONE);
            endif;

            value = %trimR(outGMonthDay);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_togMonthDay...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG gYear value.
      *=====================================================================*
     P Marshaller_togYear...
     P                 B                   export
      *
     D Marshaller_togYear...
     D                 PI         65535A          varying
     D  i_gYear                            const  like(wsdl_gYear_t      )
     D  i_timeZone                    6A   const  options(*nopass)
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Parameter positions
     D p_timeZone      C                   2
      *
      *  Helper fields
     D outGYear        DS                  likeds(xmlGYear_t      ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            outGYear.year = %editc(%dec(i_gYear: 4: 0): 'X');

            if (%parms() >= p_timeZone);
               outGYear.timeZone = getXmlTimeZone(i_timeZone);
            else;
               outGYear.timeZone = getXmlTimeZone(WSDL_LOCAL_TIMEZONE);
            endif;

            value = %trimR(outGYear);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_togYear...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG gYearMonth value.
      *=====================================================================*
     P Marshaller_togYearMonth...
     P                 B                   export
      *
     D Marshaller_togYearMonth...
     D                 PI         65535A          varying
     D  i_gYearMonth                       const  like(wsdl_gYearMonth_t )
     D  i_timeZone                    6A   const  options(*nopass)
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Parameter positions
     D p_timeZone      C                   2
      *
      *  Helper fields
     D yearMonth       DS                  qualified
     D  year                          4S 0 inz
     D  month                         2S 0 inz
     D  value                  1      6S 0
      *
     D outGYearMonth   DS                  likeds(xmlGYearMonth_t) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            yearMonth.value = i_gYearMonth;

            outGYearMonth.year      = %editc(yearMonth.year: 'X');
            outGYearMonth.delimiter = '-';
            outGYearMonth.month     = %editc(yearMonth.month: 'X');

            if (%parms() >= p_timeZone);
               outGYearMonth.timeZone = getXmlTimeZone(i_timeZone);
            else;
               outGYearMonth.timeZone = getXmlTimeZone(WSDL_LOCAL_TIMEZONE);
            endif;

            value = %trimR(outGYearMonth);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_togYearMonth...
     P                 E
      *
      *=====================================================================*
      *  Encodes the content of a given character field to Base64
      *  and returns the encoded data to the caller.
      *=====================================================================*
     P Marshaller_toBase64Binary...
     P                 B                   export
      *
     D Marshaller_toBase64Binary...
     D                 PI         65535A          varying
      /IF DEFINED(INTERNALLY_COMPILE_WSDL2R98)
     D  i_value                   65535A          varying  options(*varsize)
      /ELSE
     D  i_value                   65535A   const  varying  options(*varsize)
      /ENDIF
     D  i_ccsid                      10I 0 const  options(*nopass: *omit)
      *
      *  Return value
     D                 DS
     D value                      65535A   varying inz
     D outLength               1      2U 0
      *
      *  Parameter positions
     D p_ccsid         C                   2
      *
      *  Helper fields
     D length          S             10U 0 inz
     D inValue         S                   like(i_value) inz
     D hTranscoder     S                   like(wsdl_hTranscoder_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (%parms() >= p_ccsid and %addr(i_ccsid) <> *NULL);
               hTranscoder = Transcoder_new(i_ccsid: 0);
               inValue = Transcoder_xlateString(hTranscoder: i_value);
               Transcoder_delete(hTranscoder);
            else;
               inValue = i_value;
            endif;

            length = base64_encode(%addr(inValue)+2: %len(inValue)
                                   : %addr(value)+2: %size(value )-2);
            outLength = length;

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toBase64Binary...
     P                 E
      *
      *=====================================================================*
      *  Encodes the content of a given character field to Base64
      *  and returns the encoded data to the caller.
      *=====================================================================*
     P Marshaller_toBase64BinaryV6...
     P                 B                   export
      *
     D Marshaller_toBase64BinaryV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_length                     10I 0 const
     D  i_hOutStream                       const  like(wsdl_hStream_t)
     D  i_pWriteProc                   *   const  procptr
     D                                            options(*nopass: *omit)
     D  i_ccsid                      10I 0 const  options(*nopass: *omit)
      *
      *  Return value (not used)
     D outLength       S             10I 0 inz
      *
      *  Parameter positions
     D p_pWriteProc    C                   5
     D p_ccsid         C                   6
      *
      *  Optional parameter fields
     D pWriteProc      S                   like(i_pWriteProc) inz
      *
      *  Helper fields
     D pOutBuf         S               *   inz
     D outLen          S             10U 0 inz
     D hTranscoder     S                   like(wsdl_hTranscoder_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (%parms() >= p_ccsid and %addr(i_ccsid) <> *NULL);
               hTranscoder = Transcoder_new(i_ccsid: 0);
               outLen = Transcoder_xlateRaw(
                           hTranscoder: i_pData: i_length: pOutBuf);
               Transcoder_delete(hTranscoder);
            else;
               pOutBuf = i_pData;
               outLen = i_length;
            endif;

            if (%parms() >= p_pWriteProc and %addr(i_pWriteProc) <> *NULL);
               pWriteProc = i_pWriteProc;
            else;
               pWriteProc = getDefaultWriteProc();
            endif;

            outLength = base64_encodeBufferToStream(
                              pOutBuf: outLen: i_hOutStream: pWriteProc);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return;

      /END-FREE
      *
     P Marshaller_toBase64BinaryV6...
     P                 E
      *
      *=====================================================================*
      *  Encodes the content of a given  stream file to Base64 and
      *  writes the encoded data to the specified descriptor using
      *  the specified 'write' procedure.
      *=====================================================================*
     P Marshaller_fromFileToBase64Descriptor...
     P                 B                   export
      *
     D Marshaller_fromFileToBase64Descriptor...
     D                 PI            10I 0
     D  i_path                     5000A   const  varying  options(*varsize)
     D  i_fd                         10I 0 const
     D  i_pWrite                       *   const  procptr
      *
      *  Return value
     D size            S             10I 0 inz
      *
      *  Helper fields
     D fd              S             10I 0 inz
      *
      *  Messages
      /COPY QWSDL2RPG,USR5015                      Could not open stream file &1.
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            f_clrErrno();

            fd = open(File_fixPath(i_path): O_SHARE_RDONLY + O_RDONLY);
            if (fd = -1);
               clear USR5015;
               USR50151 = i_path;
               USR50152 = f_errno();
               USR50153 = f_strerror(f_errno());
               AppMsg_sendEscMsgAndDump(AppMsg_newError('USR5015': USR5015));
            endif;

            size = base64_encodeStreamToStream(
                         fd: %paddr('read'): i_fd: i_pWrite);

            callp close(fd);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return size;

      /END-FREE
      *
     P Marshaller_fromFileToBase64Descriptor...
     P                 E
      *
      *=====================================================================*
      *  Streams a given Base64 encoded memory buffer to the specified
      *  descriptor using the specfied 'write' procedure.
      *=====================================================================*
     P Marshaller_fromBufferToBase64Descriptor...
     P                 B                   export
      *
     D Marshaller_fromBufferToBase64Descriptor...
     D                 PI            10I 0
     D  i_pBuffer                      *   const
     D  i_size                       10U 0 const
     D  i_fd                         10I 0 const
     D  i_pWrite                       *   const  procptr
      *
      *  Return value
     D size            S             10I 0 inz
      *
      *  Helper fields
     D fd              S             10I 0 inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            size = base64_encodeBufferToStream(
                         i_pBuffer: i_size: i_fd: i_pWrite);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage('*N'));
         endmon;

         return size;

      /END-FREE
      *
     P Marshaller_fromBufferToBase64Descriptor...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG string value.
      *=====================================================================*
     P Marshaller_toLanguage...
     P                 B                   export
      *
     D Marshaller_toLanguage...
     D                 PI         65535A          varying
     D  i_value                   65535A          varying  options(*varsize)
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Helper fields
     D fromValue       DS                  likeds(wsdl_fieldv6_t )
     D toValue         DS                  likeds(wsdl_fieldv6_t )
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            fromValue.pField = %addr(i_value);
            fromValue.pData  = %addr(i_value) + 2;
            fromValue.size   = %len(i_value);

            toValue.pField = %addr(value);
            toValue.pData  = %addr(value) + 2;
            toValue.size   = %size(value) - 2;

            encodeXmlEntityToFieldV6(fromValue: toValue);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toLanguage...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG string value.
      *=====================================================================*
     P Marshaller_toLanguageV6...
     P                 B                   export
      *
     D Marshaller_toLanguageV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_length                     10I 0 const
     D  i_hOutStream                       const  like(wsdl_hStream_t)
     D  i_pWriteProc                   *   const  procptr
     D                                            options(*nopass: *omit)
      *
      *  Parameter positions
     D p_pWriteProc    C                   5
      *
      *  Optional parameter fields
     D pWriteProc      S                   like(i_pWriteProc) inz
      *
      *  Helper fields
     D fromValue       DS                  likeds(wsdl_fieldv6_t )
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (%parms() >= p_pWriteProc and %addr(i_pWriteProc) <> *NULL);
               pWriteProc = i_pWriteProc;
            else;
               pWriteProc = getDefaultWriteProc();
            endif;

            fromValue.pField = i_pField;
            fromValue.pData  = i_pData;
            fromValue.size   = i_length;

            encodeXmlEntityToStream(
                  fromValue: i_hOutStream: pWriteProc);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessageV6(i_pData: i_length));
         endmon;

         return;

      /END-FREE
      *
     P Marshaller_toLanguageV6...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG string value.
      *=====================================================================*
     P Marshaller_toToken...
     P                 B                   export
      *
     D Marshaller_toToken...
     D                 PI         65535A          varying
     D  i_value                   65535A          varying  options(*varsize)
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Helper fields
     D fromValue       DS                  likeds(wsdl_fieldv6_t )
     D toValue         DS                  likeds(wsdl_fieldv6_t )
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            fromValue.pField = %addr(i_value);
            fromValue.pData  = %addr(i_value) + 2;
            fromValue.size   = %len(i_value);

            toValue.pField = %addr(value);
            toValue.pData  = %addr(value) + 2;
            toValue.size   = %size(value) - 2;

            encodeXmlEntityToFieldV6(fromValue: toValue);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toToken...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG string value.
      *=====================================================================*
     P Marshaller_toTokenV6...
     P                 B                   export
      *
     D Marshaller_toTokenV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_length                     10I 0 const
     D  i_hOutStream                       const  like(wsdl_hStream_t)
     D  i_pWriteProc                   *   const  procptr
     D                                            options(*nopass: *omit)
      *
      *  Parameter positions
     D p_pWriteProc    C                   5
      *
      *  Optional parameter fields
     D pWriteProc      S                   like(i_pWriteProc) inz
      *
      *  Helper fields
     D fromValue       DS                  likeds(wsdl_fieldv6_t )
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (%parms() >= p_pWriteProc and %addr(i_pWriteProc) <> *NULL);
               pWriteProc = i_pWriteProc;
            else;
               pWriteProc = getDefaultWriteProc();
            endif;

            fromValue.pField = i_pField;
            fromValue.pData  = i_pData;
            fromValue.size   = i_length;

            encodeXmlEntityToStream(
                  fromValue: i_hOutStream: pWriteProc);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessageV6(i_pData: i_length));
         endmon;

         return;

      /END-FREE
      *
     P Marshaller_toTokenV6...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG string value.
      *=====================================================================*
     P Marshaller_toAnyURI...
     P                 B                   export
      *
     D Marshaller_toAnyURI...
     D                 PI         65535A          varying
     D  i_value                   65535A          varying  options(*varsize)
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Helper fields
     D fromValue       DS                  likeds(wsdl_fieldv6_t )
     D toValue         DS                  likeds(wsdl_fieldv6_t )
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            fromValue.pField = %addr(i_value);
            fromValue.pData  = %addr(i_value) + 2;
            fromValue.size   = %len(i_value);

            toValue.pField = %addr(value);
            toValue.pData  = %addr(value) + 2;
            toValue.size   = %size(value) - 2;

            encodeXmlEntityToFieldV6(fromValue: toValue);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toAnyURI...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG string value.
      *=====================================================================*
     P Marshaller_toAnyURIV6...
     P                 B                   export
      *
     D Marshaller_toAnyURIV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_length                     10I 0 const
     D  i_hOutStream                       const  like(wsdl_hStream_t)
     D  i_pWriteProc                   *   const  procptr
     D                                            options(*nopass: *omit)
      *
      *  Parameter positions
     D p_pWriteProc    C                   5
      *
      *  Optional parameter fields
     D pWriteProc      S                   like(i_pWriteProc) inz
      *
      *  Helper fields
     D fromValue       DS                  likeds(wsdl_fieldv6_t )
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (%parms() >= p_pWriteProc and %addr(i_pWriteProc) <> *NULL);
               pWriteProc = i_pWriteProc;
            else;
               pWriteProc = getDefaultWriteProc();
            endif;

            fromValue.pField = i_pField;
            fromValue.pData  = i_pData;
            fromValue.size   = i_length;

            encodeXmlEntityToStream(
                  fromValue: i_hOutStream: pWriteProc);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessageV6(i_pData: i_length));
         endmon;

         return;

      /END-FREE
      *
     P Marshaller_toAnyURIV6...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG string value.
      *=====================================================================*
     P Marshaller_toAttributedURI...
     P                 B                   export
      *
     D Marshaller_toAttributedURI...
     D                 PI         65535A          varying
     D  i_value                   65535A          varying  options(*varsize)
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Helper fields
     D fromValue       DS                  likeds(wsdl_fieldv6_t )
     D toValue         DS                  likeds(wsdl_fieldv6_t )
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            fromValue.pField = %addr(i_value);
            fromValue.pData  = %addr(i_value) + 2;
            fromValue.size   = %len(i_value);

            toValue.pField = %addr(value);
            toValue.pData  = %addr(value) + 2;
            toValue.size   = %size(value) - 2;

            encodeXmlEntityToFieldV6(fromValue: toValue);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return value;

      /END-FREE
      *
     P Marshaller_toAttributedURI...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG string value.
      *=====================================================================*
     P Marshaller_toAttributedURIV6...
     P                 B                   export
      *
     D Marshaller_toAttributedURIV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_length                     10I 0 const
     D  i_hOutStream                       const  like(wsdl_hStream_t)
     D  i_pWriteProc                   *   const  procptr
     D                                            options(*nopass: *omit)
      *
      *  Parameter positions
     D p_pWriteProc    C                   5
      *
      *  Optional parameter fields
     D pWriteProc      S                   like(i_pWriteProc) inz
      *
      *  Helper fields
     D fromValue       DS                  likeds(wsdl_fieldv6_t )
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (%parms() >= p_pWriteProc and %addr(i_pWriteProc) <> *NULL);
               pWriteProc = i_pWriteProc;
            else;
               pWriteProc = getDefaultWriteProc();
            endif;

            fromValue.pField = i_pField;
            fromValue.pData  = i_pData;
            fromValue.size   = i_length;

            encodeXmlEntityToStream(
                  fromValue: i_hOutStream: pWriteProc);

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessageV6(i_pData: i_length));
         endmon;

         return;

      /END-FREE
      *
     P Marshaller_toAttributedURIV6...
     P                 E
      *
      *=====================================================================*
      *  Returns the XML encoded value of a given RPG duration value.
      *=====================================================================*
     P Marshaller_toDuration...
     P                 B                   export
      *
     D Marshaller_toDuration...
     D                 PI         65535A          varying
     D  i_value                            const  likeds(wsdl_duration_t )
      *
      *  Return value
     D duration        S            128A   varying inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         monitor;

            if (i_value.isNegative);
               duration = '-P';
            else;
               duration = 'P';
            endif;

            duration = duration + %trim(%char(i_value.years   )) + 'Y';   // years
            duration = duration + %trim(%char(i_value.months  )) + 'M';   // months
            duration = duration + %trim(%char(i_value.days    )) + 'D';   // days

            duration = duration + 'T';

            duration = duration + %trim(%char(i_value.hours   )) + 'H';   // hours
            duration = duration + %trim(%char(i_value.minutes )) + 'M';   // minutes
            duration = duration + %trim(%char(i_value.seconds )) + 'S';   // seconds

            clearFieldIdentifier();
         on-error;
            clearFieldIdentifier();
            AppMsg_sendEscMsgAndDump(
                  produceUnexpectedErrorMessage(i_value));
         endmon;

         return duration;

      /END-FREE
      *
     P Marshaller_toDuration...
     P                 E
      *
      * ==================================================================
      * Sets the size of the variable that takes the content
      * of a XML 'schema' element.
      * ==================================================================
     P Schema_initialize...
     P                 B                   export
     D Schema_initialize...
     D                 PI
     D  i_size                       10I 0 value
      *
      *  Local fields
     D field           DS                  likeds(wsdl_fieldv6_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /free

         field.pField = *NULL;
         field.pData  = %alloc(i_size);
         field.size   = i_size;

         clear g_ns;
         g_schema.field   = field;
         g_schema.size    = 0;
         g_schema.maxSize = i_size;
         g_schema.hMemWtr = MemoryWriter_new(g_schema.field.pData: i_size);
         g_schema.isV6    = cFalse;

         return;

      /end-free
      *
     P Schema_initialize...
     P                 E
      *
      * ==================================================================
      * Sets the size of the variable that takes the content
      * of a XML 'schema' element.
      * ==================================================================
     P Schema_initializeV6...
     P                 B                   export
     D Schema_initializeV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_size                       10I 0 const
      *
      *  Local fields
     D field           DS                  likeds(wsdl_fieldv6_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /free

         field.pField = i_pField;
         field.pData  = i_pData;
         field.size   = i_size;

         clear g_ns;
         g_schema.field   = field;
         g_schema.size    = 0;
         g_schema.maxSize = getMaxLengthV6(field);
         g_schema.hMemWtr = MemoryWriter_new(
                                  g_schema.field.pData: g_schema.maxSize);
         g_schema.isV6    = cTrue;

         return;

      /end-free
      *
     P Schema_initializeV6...
     P                 E
      *
      * ==================================================================
      * Retrieves athe content of a XML 'schema' element
      * from the XML stream.
      * ==================================================================
     P Schema_start...
     P                 B                   export
     D Schema_start...
     D                 PI
     D  userdata                       *   value
     D  depth                        10I 0 value
     D  namespace                  1024A   varying const
     D  name                       1024A   varying const
     D  path                      24576A   varying const
     D  attrs                          *   dim(32767)
     D                                     const options(*varsize)
      *
     D schema          S          65535A   varying based(userdata)
      *
     D tmpSchema       S           4096A   varying inz
      *
     D qAttr           S                   like(wsdl_name_t ) inz
      *
     D attribute       DS                  likeds(attribute_t ) inz
      *
     D p               S             10I 0 inz
     D x               S             10I 0 inz
     D i               S             10I 0 inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /free

         tmpSchema = tmpSchema + '<' + cNS_XSD + ':' + name;

         if (depth = 1);
            %len(schema) = 0;
            tmpSchema = tmpSchema + ' %%ns%%';
         endif;

         if (attrs(1) = *NULL);
            tmpSchema = tmpSchema + '>';
         else;
            x = 1;
            dow (attrs(x) <> *NULL);
               clear attribute;
               qAttr = %str(attrs(x));
               p = %scan(x'0c': qAttr);
               if (p > 0);
                  attribute.ns   = %subst(qAttr: 1: p-1);
                  attribute.name = %subst(qAttr: p+1);
                  i = %lookup(attribute.ns: g_ns);
                  if (i = 0);
                     i = %lookup('': g_ns);
                     g_ns(i) = attribute.ns;
                  endif;
                  attribute.ns = cNS_PREFIX + %char(i);
               else;
                  attribute.ns   = '';
                  attribute.name = qAttr;
               endif;
               attribute.value = %str(attrs(x+1));

               if (attribute.ns = '');
                  tmpSchema = tmpSchema + ' ' +
                              attribute.name + '="' + attribute.value + '"';
               else;
                  tmpSchema = tmpSchema + ' ' +
                              attribute.ns + ':' +
                              attribute.name + '="' + attribute.value + '"';
               endif;
               x = x + 2;
            enddo;

            tmpSchema = tmpSchema + '>';
         endif;

         Buffer_append(g_schema: tmpSchema);

         return;

      /end-free
      *
     P Schema_start...
     P                 E
      *
      * ==================================================================
      * Retrieves athe content of a XML 'schema' element
      * from the XML stream.
      * ==================================================================
     P Schema_end...
     P                 B                   export
     D Schema_end...
     D                 PI
     D  userdata                       *   value
     D  depth                        10I 0 value
     D  namespace                  1024A   varying const
     D  name                       1024A   varying const
     D  path                      24576A   varying const
     D  value                     65535A   varying const
     D  attrs                          *   dim(32767)
     D                                     const options(*varsize)
      *
     D schema          S          65535A   varying based(userdata)
      *
     D maxLen          S             10I 0 inz
      *
     D i               S             10I 0 inz
     D ns              S           4096A   varying inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /free

         Buffer_append(g_schema: '</' + cNS_XSD + ':' + name + '>');

         if (depth = 1);
            ns = 'xmlns:' + cNS_XSD + '="http://www.w3.org/2001/XMLSchema"';
            i = 1;
            dow (i < %elem(g_ns) and g_ns(i) <> '');
               ns = ns + ' ' +
                    'xmlns:' + cNS_PREFIX + %char(i) + '="' + g_ns(i) + '"';
               i = i + 1;
            enddo;

            Buffer_replace(g_schema: '%%ns%%': ns);

            if (not g_schema.isV6);
               %len(schema) = g_schema.size;
               memcpy(%addr(schema)+2: g_schema.field.pData: g_schema.size);
            else;
               setLengthV6(g_schema.field: g_schema.size);
            endif;

            Schema_terminate();
         endif;

         return;

      /end-free
      *
     P Schema_end...
     P                 E
      *
      * ==================================================================
    R * *** private ***
      * Terminates 'schema' processing and returns
      * the schema to the caller.
      * ==================================================================
     P Schema_terminate...
     P                 B
     D Schema_terminate...
     D                 PI
      *
      *  Local fields
     D field           DS                  likeds(wsdl_fieldV6_t )
     D                                     based(pField)
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /free

         pField = %addr(g_schema.field);

         clear g_ns;
         if (not g_schema.isV6);
            dealloc(N) field.pData;   // = g_schema.field.pData
         endif;
         MemoryWriter_delete(g_schema.hMemWtr);
         clear g_schema;

         return;

      /end-free
      *
     P Schema_terminate...
     P                 E
      *
      * ==================================================================
      * Sets the size of the variable that takes the content
      * of a XML 'any' element.
      * ==================================================================
     P Any_initialize...
     P                 B                   export
     D Any_initialize...
     D                 PI
     D  i_size                       10I 0 value
      *
      *  Local fields
     D field           DS                  likeds(wsdl_fieldv6_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /free

         field.pField = *NULL;
         field.pData  = %alloc(i_size);
         field.size   = i_size;

         clear g_ns;
         g_any.field   = field;
         g_any.size    = 0;
         g_any.maxSize = i_size;
         g_any.hMemWtr = MemoryWriter_new(g_any.field.pData: i_size);
         g_any.isV6    = cFalse;

         return;

      /end-free
      *
     P Any_initialize...
     P                 E
      *
      * ==================================================================
      * Sets the size of the variable that takes the content
      * of a XML 'any' element.
      * ==================================================================
     P Any_initializeV6...
     P                 B                   export
     D Any_initializeV6...
     D                 PI
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_size                       10I 0 const
      *
      *  Local fields
     D field           DS                  likeds(wsdl_fieldv6_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /free

         field.pField = i_pField;
         field.pData  = i_pData;
         field.size   = i_size;

         clear g_ns;
         g_any.field   = field;
         g_any.size    = 0;
         g_any.maxSize = getMaxLengthV6(field);
         g_any.hMemWtr = MemoryWriter_new(
                               g_any.field.pData: g_any.maxSize);
         g_any.isV6    = cTrue;

         return;

      /end-free
      *
     P Any_initializeV6...
     P                 E
      *
      * ==================================================================
      * Retrieves the content of a XML 'any' element
      * from the XML stream.
      * ==================================================================
     P Any_start...
     P                 B                   export
     D Any_start...
     D                 PI
     D  userdata                       *   value
     D  depth                        10I 0 value
     D  namespace                  1024A   varying const
     D  name                       1024A   varying const
     D  path                      24576A   varying const
     D  attrs                          *   dim(32767)
     D                                     const options(*varsize)
      *
     D any             S          65535A   varying based(userdata)
      *
     D tmpAny          S           4096A   varying inz
      *
     D qName           DS                  likeds(wsdl_qName_t) inz
     D qAttr           S                   like(wsdl_name_t   ) inz
      *
     D attribute       DS                  likeds(attribute_t ) inz
      *
     D p               S             10I 0 inz
     D x               S             10I 0 inz
     D i               S             10I 0 inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /free

         if (namespace <> '');
            qName.nsValue = namespace;
            qName.lName   = name;
            i = %lookup(qName.nsValue: g_ns);
            if (i = 0);
               i = %lookup('': g_ns);
               g_ns(i) = qName.nsValue;
            endif;
            qName.nsPrefix = cNS_PREFIX + %char(i);
         else;
            qName.nsValue  = '';
            qName.lName    = name;
            qName.nsPrefix = '';
         endif;

         if (qName.nsPrefix <> '');
            tmpAny = tmpAny + '<' + qName.nsPrefix + ':' + qName.lName;
         else;
            tmpAny = tmpAny + '<' + qName.lName;
         endif;

         if (depth = 1);
            tmpAny = tmpAny + ' %%ns%%';
         endif;

         if (attrs(1) = *NULL);
            tmpAny = tmpAny + '>';
         else;
            x = 1;
            dow (attrs(x) <> *NULL);
               clear attribute;
               qAttr = %str(attrs(x));
               p = %scan(x'0c': qAttr);
               if (p > 0);
                  attribute.ns   = %subst(qAttr: 1: p-1);
                  attribute.name = %subst(qAttr: p+1);
                  i = %lookup(attribute.ns: g_ns);
                  if (i = 0);
                     i = %lookup('': g_ns);
                     g_ns(i) = attribute.ns;
                  endif;
                  attribute.ns = cNS_PREFIX + %char(i);
               else;
                  attribute.ns   = '';
                  attribute.name = qAttr;
               endif;
               attribute.value = %str(attrs(x+1));

               if (attribute.ns = '');
                  tmpAny = tmpAny + ' ' +
                           attribute.name + '="' + attribute.value + '"';
               else;
                  tmpAny = tmpAny + ' ' +
                           attribute.ns + ':' +
                           attribute.name + '="' + attribute.value + '"';
               endif;
               x = x + 2;
            enddo;

            tmpAny = tmpAny + '>';
         endif;

         Buffer_append(g_any: tmpAny);

         return;

      /end-free
      *
     P Any_start...
     P                 E
      *
      * ==================================================================
      * Retrieves the content of a XML 'any' element
      * from the XML stream.
      * ==================================================================
     P Any_end...
     P                 B                   export
     D Any_end...
     D                 PI
     D  userdata                       *   value
     D  depth                        10I 0 value
     D  namespace                  1024A   varying const
     D  name                       1024A   varying const
     D  path                      24576A   varying const
     D  value                     65535A   varying const
     D  attrs                          *   dim(32767)
     D                                     const options(*varsize)
      *
     D any             S          65535A   varying based(userdata)
      *
     D qName           DS                  likeds(wsdl_qName_t) inz
     D tmpAny          S           4096A   varying inz
      *
     D maxLen          S             10I 0 inz
      *
     D p               S             10I 0 inz
     D i               S             10I 0 inz
     D ns              S           4096A   varying inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /free

         if (value <> '');
            Buffer_append(g_any: value);
         endif;

         if (namespace <> '');
            qName.nsValue = namespace;
            qName.lName   = name;
            i = %lookup(qName.nsValue: g_ns);
            qName.nsPrefix = cNS_PREFIX + %char(i);
         else;
            qName.nsValue  = '';
            qName.lName    = name;
            qName.nsPrefix = '';
         endif;

         if (qName.nsPrefix <> '');
            tmpAny = tmpAny + '</' + qName.nsPrefix + ':' + qName.lName + '>';
         else;
            tmpAny = tmpAny + '</' + qName.lName + '>';
         endif;

         Buffer_append(g_any: tmpAny);

         if (depth = 1);
            i = 1;
            dow (i < %elem(g_ns) and g_ns(i) <> '');
               ns = ns + ' ' +
                    'xmlns:' + cNS_PREFIX + %char(i) + '="' + g_ns(i) + '"';
               i = i + 1;
            enddo;

            Buffer_replace(g_any: '%%ns%%': ns);

            if (not g_any.isV6);
               %len(any) = g_any.size;
               memcpy(%addr(any)+2: g_any.field.pData: g_any.size);
            else;
               setLengthV6(g_any.field: g_any.size);
            endif;

            Any_terminate();
         endif;

         return;

      /end-free
      *
     P Any_end...
     P                 E
      *
      * ==================================================================
    R * *** private ***
      * Terminates 'any' processing and returns
      * the schema to the caller.
      * ==================================================================
     P Any_terminate...
     P                 B
     D Any_terminate...
     D                 PI
      *
      *  Local fields
     D field           DS                  likeds(wsdl_fieldV6_t )
     D                                     based(pField)
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /free

         pField = %addr(g_any.field);

         clear g_ns;
         if (not g_any.isV6);
            dealloc(N) field.pData;   // = g_any.field.pData
         endif;
         MemoryWriter_delete(g_any.hMemWtr);
         clear g_any;

         return;

      /end-free
      *
     P Any_terminate...
     P                 E
      *
      *=====================================================================*
      *  Enables/disables the integrated XML entity encode.
      *=====================================================================*
     P XmlEntityEncoder_setEnabled...
     P                 B                   export
     D                 PI
     D  i_isEnabled                    N   const
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         g_doEncodeXmlEntities = i_isEnabled;

      /END-FREE
      *
     P                 E
      *
      *=====================================================================*
      *  Returns the enabled status of the integrated XML entity encoder.
      *=====================================================================*
     P XmlEntityEncoder_isEnabled...
     P                 B                   export
     D                 PI              N
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         return g_doEncodeXmlEntities;

      /END-FREE
      *
     P                 E
      *
      *=====================================================================*
      *  Encodes XML entities up to 64k (V5R4 and lower).
      *=====================================================================*
     P XmlEntityEncoder_encode...
     P                 B                   export
     D                 PI         65535A          varying
     D  i_value                   65535A          varying  options(*varsize)
      *
      *  Return value
     D value           S          65535A   varying inz
      *
      *  Helper fields
     D fromValue       DS                  likeds(wsdl_fieldv6_t )
     D toValue         DS                  likeds(wsdl_fieldv6_t )
     D length_2        S              5U 0 based(pLength)
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         fromValue.pField = %addr(i_value);
         fromValue.pData  = %addr(i_value) + %size(length_2);
         fromValue.size   = %len(i_value) + %size(length_2); // = equals: %size(..)

         toValue.pField = %addr(value);
         toValue.pData  = %addr(value) + %size(length_2);
         toValue.size   = %size(value);

         encodeXmlEntityToFieldV6(fromValue: toValue);

         return value;

      /END-FREE
      *
     P                 E
      *
      *=====================================================================*
      *  Encodes XML entities up to 1MB (V6R1 or higher).
      *  (Large String = 1048576 Bytes on V6R1+)
      *=====================================================================*
     P XmlEntityEncoder_encodeLarge...
     P                 B                   export
     D                 PI                         like(wsdl_largeString_t)
     D  i_value                                   like(wsdl_largeString_t)
     D                                            options(*varsize)
      *
      *  Return value
     D value           S                   like(wsdl_largeString_t) inz
      *
      *  Helper fields
     D fromValue       DS                  likeds(wsdl_fieldv6_t )
     D toValue         DS                  likeds(wsdl_fieldv6_t )
     D length_4        S             10U 0 based(pLength)
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         fromValue.pField = %addr(i_value);
         fromValue.pData  = %addr(i_value) + %size(length_4);
         fromValue.size   = %len(i_value) + %size(length_4); // = equals: %size(..)

         toValue.pField = %addr(value);
         toValue.pData  = %addr(value) + %size(length_4);
         toValue.size   = %size(value);

         encodeXmlEntityToFieldV6(fromValue: toValue);

         return value;

      /END-FREE
      *
     P                 E
      *
      *=====================================================================*
      *  Encodes XML entities up to 16MB (V6R1 or higher).
      *=====================================================================*
      *  Parameters:
      *   i_value       - String value that is encoded.
      *   i_pField      - Value of %addr(..) of the target field.
      *   i_pData       - Value of %addr(..: *DATA) of the target field.
      *   i_size        - Value of %size(..) of the target field.
      *
      *  Returns:
      *   void
      *=====================================================================*
     P XmlEntityEncoder_encodeV6...
     P                 B                   export
     D                 PI
     D  i_value                                   like(wsdl_extraLargeString_t)
     D                                            options(*varsize)
     D  i_pField                       *   const
     D  i_pData                        *   const
     D  i_size                       10I 0 const
      *
      *  Helper fields
     D fromValue       DS                  likeds(wsdl_fieldv6_t )
     D toValue         DS                  likeds(wsdl_fieldv6_t )
      /if defined(*V6R1M0)
     D length_4        S             10U 0 based(pLength)
      /else
     D length_4        S              5U 0 based(pLength)
      /endif
      *
      *  Messages
      /COPY QWSDL2RPG,USR9996                      WARNING: Procedure '&1' is not ...
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

      /if not defined(*V6R1M0)
         clear USR9996;
         USR99961 = f_this();
         AppMsg_sendCancelMsg(AppMsg_newError('USR9996': USR9996));
      /endif

         fromValue.pField = %addr(i_value);
         fromValue.pData  = %addr(i_value) + %size(length_4);
         fromValue.size   = %len(i_value) + %size(length_4); // = equals: %size(..)

         toValue.pField = i_pField;
         toValue.pData  = i_pData;
         toValue.size   = i_size;

         encodeXmlEntityToFieldV6(fromValue: toValue);

         return;

      /END-FREE
      *
     P                 E
      *
      *=====================================================================*
      *  Returns cTrue if a given list contains a 'nil' attribute, set
      *  to 'true', else cFalse.
      *=====================================================================*
     P Value_isNil...
     P                 B                   export
      *
     D Value_isNil...
     D                 PI              N
     D  i_attrs                        *   const  dim(32767) options(*varsize)
      *
      *  Return value
     D isNil           S               N   inz
      *
      *  Helper fields
     D value           S             16A   varying inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         value = XMLSAXParser_getAttributeValueByQName(
                       i_attrs: cNS_SCHEMA_INSTANCE + ':nil');

         select;
         when (value = 'true' or value = '1');
            isNil = cTrue;
         when (value = 'false' or value = '0');
            isNil = cFalse;
         other;
            isNil = cFalse;
         endsl;

         return isNil;

      /END-FREE
      *
     P Value_isNil...
     P                 E
      *
      * ==================================================================
      * Appends a given string to a given buffer of type 'schema' or 'any'.
      * ==================================================================
     P Buffer_append...
     P                 B
     D Buffer_append...
     D                 PI
     D  io_buffer                                   likeds(buffer_t )
     D  i_string                       *   const    options(*string)
      *
     D size            S             10I 0 inz
     D rc              S             10I 0 inz
      *
      *  Messages
      /COPY QWSDL2RPG,USR5022                      Buffer too small to hold result. The ...
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /free

         size = strlen(i_string);

         rc = MemoryWriter_write(io_buffer.hMemWtr: i_string: size);
         if (rc = RC_ERROR);
            clear USR5022;
            USR50221 = io_buffer.maxSize;
            AppMsg_sendEscMsgAndDump(AppMsg_newError('USR5022': USR5022));
         endif;

         io_buffer.size = io_buffer.size + rc;

         return;

      /end-free
      *
     P Buffer_append...
     P                 E
      *
      * ==================================================================
      * Replaces a given string within the specified buffer.
      * ==================================================================
     P Buffer_replace...
     P                 B
     D Buffer_replace...
     D                 PI
     D  io_buffer                                   likeds(buffer_t )
     D  i_arg                          *   const    options(*string)
     D  i_value                        *   const    options(*string)
      *
     D x               S             10I 0 inz
     D length_1        S             10I 0 inz(-1)
     D length_2        S             10I 0 inz(-1)
     D lenArg          S             10I 0 inz
     D lenValue        S             10I 0 inz
     D char            S              1A   based(pChar)
     D arg1Char        S              1A   based(i_arg)
     D pDest           S               *   inz
     D pSrc            S               *   inz
      *
      *  Messages
      /COPY QWSDL2RPG,USR5022                      Buffer too small to hold result. The ...
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /free

         lenArg   = strlen(i_arg);
         lenValue = strlen(i_value);

         if (lenArg = 0);
            return;
         endif;

         for x = 0 to (io_buffer.size - lenArg);
            pChar = io_buffer.field.pData + x;
            if (char = arg1Char);
               if (memcmp(pChar: i_arg: lenArg) = 0);
                  length_1 = x;
                  leave;
               endif;
            endif;
         endfor;

         if (length_1 = -1);
            return;
         endif;

         if (io_buffer.maxSize - io_buffer.size < lenValue - lenArg);
            clear USR5022;
            USR50221 = io_buffer.maxSize;
            AppMsg_sendEscMsgAndDump(AppMsg_newError('USR5022': USR5022));
         endif;

         length_2 = io_buffer.size - (length_1 + lenArg);

         pDest = io_buffer.field.pData + length_1 + lenValue;
         pSrc  = io_buffer.field.pData + length_1 + lenArg;
         memmove(pDest: pSrc: length_2);

         if (lenValue > 0);
            pDest = io_buffer.field.pData + length_1;
            memcpy(pDest: i_value: lenValue);
         endif;

         io_buffer.size = io_buffer.size - lenArg + lenValue;

         return;

      /end-free
      *
     P Buffer_replace...
     P                 E
      *
      *=====================================================================*
    R *  *** private ***
      *  Checks whether or not the XML encoder is enabled
      *  end encodes the XML entities of a given string.
      *=====================================================================*
     P encodeXmlEntityToFieldV6...
     P                 B
     D                 PI
     D  i_fromField                        const  likeds(wsdl_fieldv6_t )
     D  i_toField                          const  likeds(wsdl_fieldv6_t )
      *
      *  Helper fields
     D fd              S                   like(memWtr_handle_t ) inz
     D length          S             10I 0 inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         fd = MemoryWriter_new(i_toField.pData: i_toField.size);
         length = encodeXmlEntityToStream(i_fromField: fd
                        : %paddr('BASICS1RH_MemoryWriter_write'));
         MemoryWriter_delete(fd);

         // Set length of target field
         setLengthV6(i_toField: length);

         return;

      /END-FREE
      *
     P                 E
      *
      *=====================================================================*
    R *  *** private ***
      *  Checks whether or not the XML encoder is enabled
      *  end encodes the XML entities of a given string.
      *=====================================================================*
     P encodeXmlEntityToStream...
     P                 B
     D                 PI            10I 0
     D  i_fromField                        const  likeds(wsdl_fieldv6_t )
     D  i_fd                               const  like(memWtr_handle_t  )
     D  i_pWriteProc                   *   const  procptr
      *
      *  Helper fields
     D totalSize       S             10I 0 inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         if (not g_doEncodeXmlEntities);
            totalSize =
               GenericStream_write(
               i_fd: i_pWriteProc: i_fromField.pData: getLengthV6(i_fromField));
            return totalSize;
         endif;

         totalSize = encodeXmlEntityToStreamInternally(
                           i_fromField: i_fd: i_pWriteProc);

         return totalSize;

      /END-FREE
      *
     P                 E
      *
      *=====================================================================*
    R *  *** private ***
      *  Encodes XML entities.
      *=====================================================================*
     P encodeXmlEntityToStreamInternally...
     P                 B
     D                 PI            10I 0
     D  i_fromField                        const  likeds(wsdl_fieldv6_t )
     D  i_fd                               const  like(memWtr_handle_t  )
     D  i_pWriteProc                   *   const  procptr
      *
      *  Helper fields
     D x               S             10I 0 inz
     D y               S             10I 0 inz
     D fromChar        S              1A   based(pFromChar)
     D copy            DS                  qualified
     D  pFrom                          *   inz
     D  size                         10I 0 inz
     D copyDelayed     DS                  likeds(copy) inz
     D size            S             10I 0 inz
     D totalSize       S             10I 0 inz
      *
      *  Static fields
     D isInit          S               N   inz(cFalse)     static
     D esc_from        S             10A   Dim(5) varying  static
     D esc_to          S             10A   Dim(5) varying  static
     D esc_size        S             10U 0 Dim(5)          static
      *
      *  Constants
     D cESC_FROM_1     C                   U'0026'                              &
     D cESC_FROM_2     C                   U'003E'                              >
     D cESC_FROM_3     C                   U'0022'                              "
     D cESC_FROM_4     C                   U'003C'                              <
     D cESC_FROM_5     C                   U'0027'                              '
      *
     D cESC_TO_1       C                   U'00260061006D0070003B'              &amp;
     D cESC_TO_2       C                   U'002600670074003B'                  &gt;
     D cESC_TO_3       C                   U'002600710075006F0074003B'          &quot;
     D cESC_TO_4       C                   U'0026006C0074003B'                  &lt;
     D cESC_TO_5       C                   U'002600610070006F0073003B'          &apos;
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         if (not isInit);
            esc_from(1) = %char(cESC_FROM_1);
            esc_from(2) = %char(cESC_FROM_2);
            esc_from(3) = %char(cESC_FROM_3);
            esc_from(4) = %char(cESC_FROM_4);
            esc_from(5) = %char(cESC_FROM_5);
            esc_to(1)   = %char(cESC_TO_1);
            esc_to(2)   = %char(cESC_TO_2);
            esc_to(3)   = %char(cESC_TO_3);
            esc_to(4)   = %char(cESC_TO_4);
            esc_to(5)   = %char(cESC_TO_5);
            esc_size(1) = %len(esc_to(1));
            esc_size(2) = %len(esc_to(2));
            esc_size(3) = %len(esc_to(3));
            esc_size(4) = %len(esc_to(4));
            esc_size(5) = %len(esc_to(5));
            isInit = cTrue;
         endif;

         copyDelayed.pFrom = i_fromField.pData;
         copyDelayed.size  = 0;

         copy.pFrom = *NULL;
         copy.size  = 0;

         for x = 1 to getLengthV6(i_fromField);
            pFromChar  = i_fromField.pData + (x-1);

            // Check whether or not the current character
            // needs to be escaped.
            y = %lookup(fromChar: esc_from);
            if (y > 0);
               copy.pFrom = %addr(esc_to(y))+2;
               copy.size  = esc_size(y);
            endif;

            if (copy.size = 0);
               copyDelayed.size  += %size(fromChar);
            else;

               // Copy delayed bytes at once
               if (copyDelayed.size > 0);
                  size = GenericStream_write(i_fd: i_pWriteProc
                                             : copyDelayed.pFrom
                                             : copyDelayed.size);
                  totalSize = totalSize + size;
               endif;

               // Copy escaped charcater
               size = GenericStream_write(i_fd: i_pWriteProc
                                          : copy.pFrom: copy.size);
               totalSize = totalSize + size;

               copy.pFrom = *NULL;
               copy.size  = 0;

               copyDelayed.pFrom = pFromChar + 1;
               copyDelayed.size  = 0;
            endif;
         endfor;

         // Copy remaining delayed bytes at once
         if (copyDelayed.size > 0);
            size = GenericStream_write(i_fd: i_pWriteProc
                                       : copyDelayed.pFrom: copyDelayed.size);
            totalSize = totalSize + size;
         endif;

         return totalSize;

      /END-FREE
      *
     P                 E
      *
      *=====================================================================*
    R *  *** private ***
      *  Encodes XML entities.
      *=====================================================================*
     P GenericStream_write...
     P                 B
      *
     D GenericStream_write...
     D                 PI            10I 0
     D  i_fd                               const  like(memWtr_handle_t  )
     D  i_pWriteProc                   *   const  procptr
     D  i_pBuffer                      *   const                                = *buf
     D  i_buflen                           const like(size_t    )               = size_t
      *
      *  Helper fields
     D size            S             10I 0 inz
      *
      *  Generic 'write' procedure
     D Generic_write...
     D                 PR                        like(ssize_t   )
     D                                     extproc(i_pWriteProc)
     D  i_fd                         10I 0 value                                = int
     D  i_pBuffer                      *   value                                = *buf
     D  i_buflen                           value like(size_t    )               = size_t
      *
      *  Messages
      /COPY QWSDL2RPG,USR5018                      Failed marshalling a given string ...
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         if (i_buflen = 0);
            return 0;
         endif;

         size = Generic_write(i_fd: i_pBuffer: i_buflen);
         if (size = -1);
            clear USR5018;
            USR50181 = f_errno();
            USR50182 = f_strerror(f_errno());
            AppMsg_sendEscMsgAndDump(AppMsg_new('USR5018': USR5018));
         endif;

         return size;

      /END-FREE
      *
     P GenericStream_write...
     P                 E
      *
      *=====================================================================*
    R *  *** private ***
      *  Returns the length of a varying field
      *=====================================================================*
     P getLengthV6...
     P                 B
      *
     D getLengthV6...
     D                 PI            10U 0
     D  i_field                            const  likeds(wsdl_fieldv6_t )
      *
      *  Return value
     D length          S             10U 0 inz
      *
      *  Helper fields
     D length_2        S              5U 0 based(pLength)
     D length_4        S             10U 0 based(pLength)
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         // Get length of data
         pLength  = i_field.pField;
         if (isBigStringV6(i_field));
            length = length_4;
         else;
            length = length_2;
         endif;

         return length;

      /END-FREE
      *
     P getLengthV6...
     P                 E
      *
      *=====================================================================*
    R *  *** private ***
      *  Returns the length of a varying field
      *=====================================================================*
     P getMaxLengthV6...
     P                 B
      *
     D getMaxLengthV6...
     D                 PI            10U 0
     D  i_field                            const  likeds(wsdl_fieldv6_t )
      *
      *  Return value
     D length          S             10U 0 inz
      *
      *  Helper fields
     D length_2        S              5U 0 based(pLength)
     D length_4        S             10U 0 based(pLength)
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         // Get length of data
         pLength  = i_field.pField;
         if (isBigStringV6(i_field));
            length = i_field.size - %size(length_4);
         else;
            length = i_field.size - %size(length_2);
         endif;

         return length;

      /END-FREE
      *
     P getMaxLengthV6...
     P                 E
      *
      *=====================================================================*
    R *  *** private ***
      *  Sets the length of a given varying field
      *=====================================================================*
     P setLengthV6...
     P                 B
      *
     D setLengthV6...
     D                 PI
     D  i_field                            const  likeds(wsdl_fieldv6_t )
     D  i_length                     10U 0 const
      *
      *  Helper fields
     D length_2        S              5U 0 based(pLength)
     D length_4        S             10U 0 based(pLength)
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         // Get length of data
         pLength  = i_field.pField;
         if (isBigStringV6(i_field));
            length_4 = i_length;
         else;
            length_2 = i_length;
         endif;

         return;

      /END-FREE
      *
     P setLengthV6...
     P                 E
      *
      *=====================================================================*
    R *  *** private ***
      *  Returns cTrue for BigString fields.
      *=====================================================================*
     P isBigStringV6...
     P                 B
      *
     D isBigStringV6...
     D                 PI              N
     D  i_field                            const  likeds(wsdl_fieldv6_t )
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         // Determine field type: 2-byte or 4-byte length
         return  (i_field.pData - i_field.pField = 4);

      /END-FREE
      *
     P isBigStringV6...
     P                 E
      *
      *=====================================================================*
    R *  *** private ***
      *  Returns cTrue for VARYING fields.
      *=====================================================================*
     P isVaryingV6...
     P                 B
      *
     D isVaryingV6...
     D                 PI              N
     D  i_field                            const  likeds(wsdl_fieldv6_t )
     D  o_msg                                     likeds(msg_t          )
      *
      *  Messages
      /COPY QWSDL2RPG,USR5019                      Unsupported field type. Field must be ...
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         if (i_field.pData = i_field.pField);
            clear USR5019;
            o_msg = AppMsg_newError('USR5019': USR5019);
            return cFalse;
         endif;

         o_msg = AppMsg_null();
         return cTrue;

      /END-FREE
      *
     P isVaryingV6...
     P                 E
      *
      *=====================================================================*
    R *  *** private ***
      *  Returns cTrue for VARYING fields.
      *=====================================================================*
     P isTooShortV6...
     P                 B
      *
     D isTooShortV6...
     D                 PI              N
     D  i_field                            const  likeds(wsdl_fieldv6_t )
     D  i_length                     10I 0 const
     D  o_msg                                     likeds(msg_t          )
      *
      *  Messages
      /COPY QWSDL2RPG,USR5020                      Field overflow. Field is too short to ...
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         if (getMaxLengthV6(i_field) < i_length);
            clear USR5020;
            USR50201 = getMaxLengthV6(i_field);
            USR50202 = i_length;
            USR50203 = getFieldNameV6();
            o_msg = AppMsg_newError('USR5020': USR5020);
            return cTrue;
         endif;

         o_msg = AppMsg_null();
         return cFalse;

      /END-FREE
      *
     P isTooShortV6...
     P                 E
      *
      *=====================================================================*
    R *  *** private ***
      *  Returns the name of the current field.
      *=====================================================================*
     P getFieldNameV6...
     P                 B
      *
     D getFieldNameV6...
     D                 PI          1024A          varying
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         if (g_fieldIdentifier.isSet);
            return '*N';
         endif;

         return g_fieldIdentifier.name;

      /END-FREE
      *
     P getFieldNameV6...
     P                 E
      *
      *=====================================================================*
    R *  *** private ***
      *  Returns cTrue for VARYING fields.
      *=====================================================================*
     P copyValueV6...
     P                 B
      *
     D copyValueV6...
     D                 PI              N
     D  i_field                            const  likeds(wsdl_fieldv6_t )
     D  i_pValue                       *   const
     D  i_length                     10I 0 const
     D  o_msg                                     likeds(msg_t          )
      *
      *  Local fields
     D pDest           S               *   inz
      *
      *  Messages
      /COPY QWSDL2RPG,USR5021                      Error: memcpy() failed copying data: ...
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         pDest = memcpy(i_field.pData: i_pValue: i_length);
         if (pDest <> i_field.pData);
            clear USR5021;
            USR50211 = f_errno();
            USR50212 = f_strerror(f_errno());
            o_msg = AppMsg_newError('USR5021': USR5021);
            return cFalse;
         endif;

         o_msg = AppMsg_null();
         return cTrue;

      /END-FREE
      *
     P copyValueV6...
     P                 E
      *
      *=====================================================================*
    R *  *** private ***
      *  Adjust remote timestamp to local time zone.
      *=====================================================================*
     P adjustTimestamp...
     P                 B
      *
     D adjustTimestamp...
     D                 PI              Z
     D  i_xmlTimestamp...
     D                                     const  likeds(xmlTimestamp_t )
      *
      *  Return value
     D timestamp       S               Z   inz
      *
     D hours           S              5I 0 inz
     D minutes         S              5I 0 inz
      *
     D tmpTimestamp    DS                  qualified
     D  value                        20A   inz
     D  mSecs                         6A   inz(*ALL'0')
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         tmpTimestamp = i_xmlTimestamp;
         tmpTimestamp = %xlate('T' : '-' : tmpTimestamp);
         tmpTimestamp = %xlate(':' : '.' : tmpTimestamp);
         tmpTimestamp = %xlate(' ' : '0' : tmpTimestamp);

         // Adjust timestamp to GMT date
         select;
         when (i_xmlTimestamp.timeZone.ID = 'Z');          // UTC time
            hours   = 0;
            minutes = 0;
         when (i_xmlTimestamp.timeZone.ID = '-');          // negative time offset
            hours   = f_val(i_xmlTimestamp.timeZone.hours  );
            minutes = f_val(i_xmlTimestamp.timeZone.minutes);
         when (i_xmlTimestamp.timeZone.ID = '+');          // positive time offset
            hours   = f_val(i_xmlTimestamp.timeZone.hours  ) * -1;
            minutes = f_val(i_xmlTimestamp.timeZone.minutes) * -1;
         other;
            hours   = 0;
            minutes = 0;
         endsl;

         // Adjust timestamp to local date
         timestamp = mergeWithLocalTimeOffset(
                           %timestamp(tmpTimestamp): hours: minutes);

         return timestamp;

      /END-FREE
      *
     P adjustTimestamp...
     P                 E
      *
      *=====================================================================*
    R *  *** private ***
      *  Adjust remote date to local time zone.
      *=====================================================================*
     P adjustDate...
     P                 B
      *
     D adjustDate...
     D                 PI              D
     D  i_xmlDate                          const  likeds(xmlDate_t )
      *
      *  Return value
     D date            S               D   inz
      *
      *  Helper fields
     D timestamp       S               Z   inz
      *
     D hours           S              5I 0 inz
     D minutes         S              5I 0 inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         // Adjust date to GMT date
         select;
         when (i_xmlDate.timeZoneID = 'Z');                // UTC time
            hours   = 0;
            minutes = 0;
         when (i_xmlDate.timeZoneID = '-');                // negative time offset
            hours   = f_val(i_xmlDate.hours  );
            minutes = f_val(i_xmlDate.minutes);
         when (i_xmlDate.timeZoneID = '+');                // positive time offset
            hours   = f_val(i_xmlDate.hours  ) * -1;
            minutes = f_val(i_xmlDate.minutes) * -1;
         other;
            hours   = 0;
            minutes = 0;
         endsl;

         // Adjust date to local date
         timestamp = mergeWithLocalTimeOffset(
                           %date(i_xmlDate.value) + %time(): hours: minutes);

         date = %date(timestamp);

         return date;

      /END-FREE
      *
     P adjustDate...
     P                 E
      *
      *=====================================================================*
    R *  *** private ***
      *  Adjust remote time to local time zone.
      *=====================================================================*
     P adjustTime...
     P                 B
      *
     D adjustTime...
     D                 PI              T
     D  i_xmlTime                          const  likeds(xmlTimeShort_t)
      *
      *  Return value
     D time            S               T   inz
      *
      *  Helper fields
     D timestamp       S               Z   inz
      *
     D hours           S              5I 0 inz
     D minutes         S              5I 0 inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         // Adjust time to GMT date
         select;
         when (i_xmlTime.timeZone.ID = 'Z');               // UTC time
            hours   = 0;
            minutes = 0;
         when (i_xmlTime.timeZone.ID = '-');               // negative time offset
            hours   = f_val(i_xmlTime.timeZone.hours  );
            minutes = f_val(i_xmlTime.timeZone.minutes);
         when (i_xmlTime.timeZone.ID = '+');               // positive time offset
            hours   = f_val(i_xmlTime.timeZone.hours  ) * -1;
            minutes = f_val(i_xmlTime.timeZone.minutes) * -1;
         other;
            hours   = 0;
            minutes = 0;
         endsl;

         // Adjust time to local time
         timestamp = mergeWithLocalTimeOffset(
                           %date() + %time(i_xmlTime.value): hours: minutes);

         time = %time(timestamp);

         return time;

      /END-FREE
      *
     P adjustTime...
     P                 E
      *
      *=====================================================================*
    R *  *** private ***
      *  Adjust remote gDay to local time zone.
      *=====================================================================*
     P adjustGDay...
     P                 B
      *
     D adjustGDay...
     D                 PI                         like(wsdl_gDay_t      )
     D  i_xmlGDay                          const  likeds(xmlGDay_t      )
      *
      *  Return value
     D day             S                   like(wsdl_gDay_t    ) inz
      *
      *  Helper fields
     D timestamp       S               Z   inz
     D tmpTimestamp    DS                  likeds(timestamp_t ) inz
      *
     D hours           S              5I 0 inz
     D minutes         S              5I 0 inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         // Adjust gDay to GMT date
         select;
         when (i_xmlGDay.timeZone.ID = 'Z');               // UTC time
            hours   = 0;
            minutes = 0;
         when (i_xmlGDay.timeZone.ID = '-');               // negative time offset
            hours   = f_val(i_xmlGDay.timeZone.hours  );
            minutes = f_val(i_xmlGDay.timeZone.minutes);
         when (i_xmlGDay.timeZone.ID = '+');               // positive time offset
            hours   = f_val(i_xmlGDay.timeZone.hours  ) * -1;
            minutes = f_val(i_xmlGDay.timeZone.minutes) * -1;
         other;
            hours   = 0;
            minutes = 0;
         endsl;

         // Adjust time to local time
         tmpTimestamp.value = %timestamp();
         tmpTimestamp.day   = i_xmlGDay.day;

         timestamp = mergeWithLocalTimeOffset(
                           tmpTimestamp.value: hours: minutes);

         day = %subdt(timestamp: *DAYS);

         return day;

      /END-FREE
      *
     P adjustGDay...
     P                 E
      *
      *=====================================================================*
    R *  *** private ***
      *  Adjust remote gMonth to local time zone.
      *=====================================================================*
     P adjustGMonth...
     P                 B
      *
     D adjustGMonth...
     D                 PI                         like(wsdl_gMonth_t    )
     D  i_xmlGMonth                        const  likeds(xmlGMonth_t    )
      *
      *  Return value
     D month           S                   like(wsdl_gMonth_t  ) inz
      *
      *  Helper fields
     D timestamp       S               Z   inz
     D tmpTimestamp    DS                  likeds(timestamp_t ) inz
      *
     D hours           S              5I 0 inz
     D minutes         S              5I 0 inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         // Adjust gMonth to GMT date
         select;
         when (i_xmlGMonth.timeZone.ID = 'Z');               // UTC time
            hours   = 0;
            minutes = 0;
         when (i_xmlGMonth.timeZone.ID = '-');               // negative time offset
            hours   = f_val(i_xmlGMonth.timeZone.hours  );
            minutes = f_val(i_xmlGMonth.timeZone.minutes);
         when (i_xmlGMonth.timeZone.ID = '+');               // positive time offset
            hours   = f_val(i_xmlGMonth.timeZone.hours  ) * -1;
            minutes = f_val(i_xmlGMonth.timeZone.minutes) * -1;
         other;
            hours   = 0;
            minutes = 0;
         endsl;

         // Adjust time to local time
         tmpTimestamp.value = %timestamp();
         tmpTimestamp.month = i_xmlGMonth.month;

         timestamp = mergeWithLocalTimeOffset(
                           tmpTimestamp.value: hours: minutes);

         month = %subdt(timestamp: *MONTHS);

         return month;

      /END-FREE
      *
     P adjustGMonth...
     P                 E
      *
      *=====================================================================*
    R *  *** private ***
      *  Adjust remote gMonthDay to local time zone.
      *=====================================================================*
     P adjustGMonthDay...
     P                 B
      *
     D adjustGMonthDay...
     D                 PI                         like(wsdl_gMonthDay_t )
     D  i_xmlGMonthDay...
     D                                     const  likeds(xmlGMonthDay_t )
      *
      *  Return value
     D monthDay        S                   like(wsdl_gMonthDay_t  ) inz
      *
      *  Helper fields
     D timestamp       S               Z   inz
     D tmpTimestamp    DS                  likeds(timestamp_t ) inz
      *
     D hours           S              5I 0 inz
     D minutes         S              5I 0 inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         // Adjust gMonthDay to GMT date
         select;
         when (i_xmlGMonthDay.timeZone.ID = 'Z');               // UTC time
            hours   = 0;
            minutes = 0;
         when (i_xmlGMonthDay.timeZone.ID = '-');               // negative time offset
            hours   = f_val(i_xmlGMonthDay.timeZone.hours  );
            minutes = f_val(i_xmlGMonthDay.timeZone.minutes);
         when (i_xmlGMonthDay.timeZone.ID = '+');               // positive time offset
            hours   = f_val(i_xmlGMonthDay.timeZone.hours  ) * -1;
            minutes = f_val(i_xmlGMonthDay.timeZone.minutes) * -1;
         other;
            hours   = 0;
            minutes = 0;
         endsl;

         // Adjust time to local time
         tmpTimestamp.value = %timestamp();
         tmpTimestamp.month = i_xmlGMonthDay.month;
         tmpTimestamp.day   = i_xmlGMonthDay.day;

         timestamp = mergeWithLocalTimeOffset(
                           tmpTimestamp.value: hours: minutes);

         monthDay = %subdt(timestamp: *MONTHS) * 100 +
                    %subdt(timestamp: *DAYS);

         return monthDay;

      /END-FREE
      *
     P adjustGMonthDay...
     P                 E
      *
      *=====================================================================*
    R *  *** private ***
      *  Adjust remote gYear to local time zone.
      *=====================================================================*
     P adjustGYear...
     P                 B
      *
     D adjustGYear...
     D                 PI                         like(wsdl_gYear_t     )
     D  i_xmlGYear                         const  likeds(xmlGYear_t     )
      *
      *  Return value
     D year            S                   like(wsdl_gYear_t   ) inz
      *
      *  Helper fields
     D timestamp       S               Z   inz
     D tmpTimestamp    DS                  likeds(timestamp_t ) inz
      *
     D hours           S              5I 0 inz
     D minutes         S              5I 0 inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         // Adjust gYear to GMT date
         select;
         when (i_xmlGYear.timeZone.ID = 'Z');               // UTC time
            hours   = 0;
            minutes = 0;
         when (i_xmlGYear.timeZone.ID = '-');               // negative time offset
            hours   = f_val(i_xmlGYear.timeZone.hours  );
            minutes = f_val(i_xmlGYear.timeZone.minutes);
         when (i_xmlGYear.timeZone.ID = '+');               // positive time offset
            hours   = f_val(i_xmlGYear.timeZone.hours  ) * -1;
            minutes = f_val(i_xmlGYear.timeZone.minutes) * -1;
         other;
            hours   = 0;
            minutes = 0;
         endsl;

         // Adjust time to local time
         tmpTimestamp.value = %timestamp();
         tmpTimestamp.year  = i_xmlGYear.year;

         timestamp = mergeWithLocalTimeOffset(
                           tmpTimestamp.value: hours: minutes);

         year = %subdt(timestamp: *YEARS);

         return year;

      /END-FREE
      *
     P adjustGYear...
     P                 E
      *
      *=====================================================================*
    R *  *** private ***
      *  Adjust remote gYearMonth to local time zone.
      *=====================================================================*
     P adjustGYearMonth...
     P                 B
      *
     D adjustGYearMonth...
     D                 PI                         like(wsdl_gYearMonth_t)
     D  i_xmlGYearMonth...
     D                                     const  likeds(xmlGYearMonth_t)
      *
      *  Return value
     D yearMonth       S                   like(wsdl_gYearMonth_t ) inz
      *
      *  Helper fields
     D timestamp       S               Z   inz
     D tmpTimestamp    DS                  likeds(timestamp_t ) inz
      *
     D hours           S              5I 0 inz
     D minutes         S              5I 0 inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         // Adjust gYearMonth to GMT date
         select;
         when (i_xmlGYearMonth.timeZone.ID = 'Z');               // UTC time
            hours   = 0;
            minutes = 0;
         when (i_xmlGYearMonth.timeZone.ID = '-');               // negative time offset
            hours   = f_val(i_xmlGYearMonth.timeZone.hours  );
            minutes = f_val(i_xmlGYearMonth.timeZone.minutes);
         when (i_xmlGYearMonth.timeZone.ID = '+');               // positive time offset
            hours   = f_val(i_xmlGYearMonth.timeZone.hours  ) * -1;
            minutes = f_val(i_xmlGYearMonth.timeZone.minutes) * -1;
         other;
            hours   = 0;
            minutes = 0;
         endsl;

         // Adjust time to local time
         tmpTimestamp.value = %timestamp();
         tmpTimestamp.year  = i_xmlGYearMonth.year;
         tmpTimestamp.month = i_xmlGYearMonth.month;

         timestamp = mergeWithLocalTimeOffset(
                           tmpTimestamp.value: hours: minutes);

         yearMonth = %subdt(timestamp: *YEARS) * 100 +
                     %subdt(timestamp: *MONTHS);

         return yearMonth;

      /END-FREE
      *
     P adjustGYearMonth...
     P                 E
      *
      *=====================================================================*
    R *  *** private ***
      *  Merge remote hours and minutes with local time zone.
      *=====================================================================*
     P mergeWithLocalTimeOffset...
     P                 B
      *
     D mergeWithLocalTimeOffset...
     D                 PI              Z
     D  i_timestamp                    Z   const
     D  i_hours                       5I 0 const
     D  i_minutes                     5I 0 const
      *
     D timestamp       S               Z   inz
      *
     D hours           S              5I 0 inz
     D minutes         S              5I 0 inz
      *
     D utcOffset       DS                  likeds(utcOffset_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         utcOffset = f_rtvSysval_char('QUTCOFFSET');

         select;
         when (utcOffset.timeZoneID = 'Z');
            // nothing to do here
         when (utcOffset.timeZoneID = '-');   // negative time offset
            hours   = i_hours   - f_val(utcOffset.hours);
            minutes = i_minutes - f_val(utcOffset.minutes);
         when (utcOffset.timeZoneID = '+');   // negative time offset
            hours   = i_hours   + f_val(utcOffset.hours);
            minutes = i_minutes + f_val(utcOffset.minutes);
         other;
            // nothing to do here
         endsl;

         timestamp = i_timestamp + %hours(hours) + %minutes(minutes);

         return timestamp;

      /END-FREE
      *
     P mergeWithLocalTimeOffset...
     P                 E
      *
      *=====================================================================*
      *  Parses a given XML time value.
      *=====================================================================*
      *  Sample XML time values:   09:00:00
      *                            09:30:10Z          UTC time zone
      *                            09:30:10-06:00     offset from UTC time
      *                            09:30:10+06:00     offset from UTC time
      *=====================================================================*
     P parseXmlTime...
     P                 B
      *
     D parseXmlTime...
     D                 PI                         likeds(xmlTimeShort_t)
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D timeShort       DS                  likeds(xmlTimeShort_t) inz
      *
      *  Helper fields
     D x               S             10I 0 inz
     D state           S             10I 0 inz(UNKNOWN)
     D tLen            S             10I 0 inz
     D tValue          S             18A   varying inz
     D char            S              1A   inz
     D timeLong        DS                  likeds(xmlTimeLong_t ) inz
      *
     D NUMERIC         C                   '0123456789'
      *
      *  States
     D UNKNOWN         C                   0
     D HOURS           C                   1
     D MINUTES         C                   2
     D SECONDS         C                   3
     D MSECS           C                   4
     D TZ_HOURS        C                   6
     D TZ_MINUTES      C                   7
      *
      /COPY QWSDL2RPG,USR5017                       Invalid XML time value: '&1'
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         // Prepare error message
         clear USR5017;
         USR50171 = i_value;

         // Parse XML time value
         for x = 1 to %len(i_value);

            char = %subst(i_value: x: 1);

            select;
            when (state = UNKNOWN);
               if (%check(NUMERIC: char) <> 0);
                  AppMsg_sendEscMsgAndDump(AppMsg_new('USR5017': USR5017));
               endif;
               state  = HOURS;
               tValue = char;
               tLen   = 1;

            when (state = HOURS or state = MINUTES or
                  state = TZ_HOURS or state = TZ_MINUTES);
               if (char = ':');
                  if (tLen < 2);
                     AppMsg_sendEscMsgAndDump(AppMsg_new('USR5017': USR5017));
                  endif;
                  tValue = tValue + '.';
                  tLen   = 0;
                  state  = state + 1;
               else;
                  if (%check(NUMERIC: char) <> 0 or tLen >= 2);
                     AppMsg_sendEscMsgAndDump(AppMsg_new('USR5017': USR5017));
                  endif;
                  tValue = tValue + char;
                  tLen   = tLen + 1;
               endif;

            when (state = SECONDS);
               if (char = '.' or isStartOfTimezone(char));
                  if (tLen < 2);
                     AppMsg_sendEscMsgAndDump(AppMsg_new('USR5017': USR5017));
                  endif;
                  if (char = '.');
                     tValue = tValue + char;
                     state  = MSECS;
                  else;
                     tValue = tValue + '.000' + char;
                     state  = TZ_HOURS;   // hours of timezone portion
                  endif;
                  tLen = 0;
               else;
                  if (%check(NUMERIC: char) <> 0 or tLen >= 2);
                     AppMsg_sendEscMsgAndDump(AppMsg_new('USR5017': USR5017));
                  endif;
                  tValue = tValue + char;
                  tLen   = tLen + 1;
               endif;

            when (state = MSECS);
               if (isStartOfTimezone(char));
                  if (tLen = 0);
                     AppMsg_sendEscMsgAndDump(AppMsg_new('USR5017': USR5017));
                  endif;
                  tValue = tValue + f_space(3 - tLen: '0') + char;
                  tLen   = 0;
                  state  = TZ_HOURS;
               else;
                  if (%check(NUMERIC: char) <> 0 or tLen >= 3);
                     AppMsg_sendEscMsgAndDump(AppMsg_new('USR5017': USR5017));
                  endif;
                  tValue = tValue + char;
                  tLen   = tLen + 1;
               endif;

            other;
               AppMsg_sendEscMsgAndDump(AppMsg_new('USR5017': USR5017));

            endsl;

         endfor;

         timeLong = tValue;

         timeShort.value.hours   = timeLong.value.hours;
         timeShort.value.dot_1   = timeLong.value.dot_1;
         timeShort.value.minutes = timeLong.value.minutes;
         timeShort.value.dot_2   = timeLong.value.dot_2;
         timeShort.value.seconds = timeLong.value.seconds;
         timeShort.timeZone      = timeLong.timeZone;

         return timeShort;

      /END-FREE
      *
     P parseXmlTime...
     P                 E
      *
      *=====================================================================*
      *  Returns the time zone value based on the input time zone.
      *=====================================================================*
     P getXmlTimeZone...
     P                 B
      *
     D getXmlTimeZone...
     D                 PI                         likeds(xmlTimeZone_t )
     D  i_timeZone                    6A   const  options(*nopass)
      *
      *  Return value
     D xmlTimeZone     DS                  likeds(xmlTimeZone_t ) inz
      *
      *  Helper fields
     D char            S              1A   inz
     D char2           S              2A   inz
     D timeZone        S                   like(i_timeZone ) inz
      *
     D NUMERIC         C                   '0123456789'
      *
      /COPY QWSDL2RPG,USR5005                      Time zone indicator &1 does not match ...
      /COPY QWSDL2RPG,USR5006                      Invalid time zone hours: &1
      /COPY QWSDL2RPG,USR5007                      Invalid time zone minutes: &1
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         select;
         when (i_timeZone = WSDL_NO_TIMEZONE);
            return '';
         when (i_timeZone = WSDL_LOCAL_TIMEZONE);
            timeZone = f_rtvSysval_char('QUTCOFFSET');
         other;
            timeZone = i_timeZone;
         endsl;

         // Get and verify time zone indicator
         char = %subst(timeZone: 1: 1);
         if (not isStartOfTimezone(char));
            clear USR5005;
            USR50051 = char;
            AppMsg_sendEscMsgAndDump(AppMsg_new('USR5005': USR5005));
         endif;

         xmlTimeZone.ID = char;

         if (xmlTimeZone.ID = 'Z');
            return xmlTimeZone;   // UTC time
         endif;

         // The next two bytes are supposed to be the hours and must be numeric
         char2 = %subst(timeZone: 2: 2);
         if (%check(NUMERIC: char2) <> 0);
            clear USR5006;
            USR50061 = char2;
            AppMsg_sendEscMsgAndDump(AppMsg_new('USR5006': USR5006));
         endif;

         xmlTimeZone.hours = char2;

         // Distinguish between the iSeries system value format and the XML format
         char = %subst(timeZone: 4: 1);
         if (char = ':');
            xmlTimeZone.delimiter = char;        // XML time zone format
            char2 = %subst(timeZone: 5: 2);
         else;
            xmlTimeZone.delimiter = ':';         // iSeries system value format
            char2 = %subst(timeZone: 4: 2);
         endif;

         if (%check(NUMERIC: char2) <> 0);
            clear USR5007;
            USR50071 = char2;
            AppMsg_sendEscMsgAndDump(AppMsg_new('USR5007': USR5007));
         endif;

         xmlTimeZone.minutes = char2;

         return xmlTimeZone;

      /END-FREE
      *
     P getXmlTimeZone...
     P                 E
      *
      *=====================================================================*
      *  Returns TRUE if a given character indicates the start of the
      *  timezone portion of a date/time value.
      *=====================================================================*
     P isStartOfTimezone...
     P                 B
      *
     D isStartOfTimezone...
     D                 PI              N
     D  i_char                        1A   const
      *
      *  Return value
     D isValid         DS                  likeds(xmlTimeZone_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         if (i_char = '-' or i_char = '+' or i_char = 'Z');
            return cTrue;
         endif;

         return cFalse;

      /END-FREE
      *
     P isStartOfTimezone...
     P                 E
      *
      *=====================================================================*
      *  Returns TRUE if a given timezone matches the specification.
      *=====================================================================*
     P isValidTimezone...
     P                 B
      *
     D isValidTimezone...
     D                 PI              N
     D  i_timeZone                         const  likeds(xmlTimeZone_t )
      *
      *  Return value
     D isValid         DS                  likeds(xmlTimeZone_t ) inz
      *
     D NUMERIC         C                   '0123456789'
      *
     D int             S             10I 0 inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         if (i_timezone.ID <> ' ' and i_timezone.ID <> 'Z' and
             i_timezone.ID <> '+' and i_timezone.ID <> '-');
            return cFalse;
         endif;

         if (i_timezone.ID = ' ' or i_timezone.ID = 'Z');
            if (i_timezone.hours <> '' or
                i_timezone.delimiter <> '' or
                i_timezone.minutes <> '');
               return cFalse;
            endif;
         endif;

         if (i_timezone.ID <> ' ');
            if (i_timezone.delimiter <> ':');
               return cFalse;
            endif;

            if (%check(NUMERIC: i_timezone.hours) <> 0);
               return cFalse;
            endif;

            if (%check(NUMERIC: i_timezone.minutes) <> 0);
               return cFalse;
            endif;

            int = %dec(i_timezone.hours: 10: 0);
            if (int < 0 or int > 24);
               return cFalse;
            endif;

            int = %dec(i_timezone.minutes: 10: 0);
            if (int < 0 or int > 60);
               return cFalse;
            endif;
         endif;

         return cTrue;

      /END-FREE
      *
     P isValidTimezone...
     P                 E
      *
      *=====================================================================*
      *  Edits a decimal value removing trailing zeros.
      *=====================================================================*
     P editDecimal...
     P                 B
      *
     D editDecimal...
     D                 PI            64A          varying
     D  i_decimal                          const  like(wsdl_decimal_t )
      *
      *  Return value
     D string          S             64A   varying inz
      *
      *  Helper fields
     D i               S             10I 0 inz
     D c               S             10I 0 inz
      *
      *  Helper fields, static
     D isInit          S               N   inz(cFalse) static
     D decComma        S              1A   inz         static
     D DECIMAL         S              1P 1 inz         static
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         if (i_decimal = 0);
            return '0';
         endif;

         string = %trim(%editc(i_decimal: 'P'));

         if (not isInit);
            decComma = %subst(%editc(DECIMAL: 'L'): 1: 1);
            isInit = cTrue;
         endif;

         c = %scan(decComma: string);
         select;
         when (c = 0);
            return string;
         when (c = 1);
            return '0';
         endsl;

         i = %len(string);
         dow (i > c and %subst(string: i: 1) = '0');
            i = i - 1;
         enddo;

         if (i = c);
            string = %subst(string: 1: c-1);
         else;
            string = %subst(string: 1: i);
         endif;

         return string;

      /END-FREE
      *
     P editDecimal...
     P                 E
      *
      *=====================================================================*
      *  Returns the default 'write' procedure used to write marshalled data.
      *=====================================================================*
     P getDefaultWriteProc...
     P                 B
      *
     D getDefaultWriteProc...
     D                 PI              *          procptr
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         return %paddr('WSDL2R87_ManagedMemoryDataSource_OutputStream_write');

      /END-FREE
      *
     P getDefaultWriteProc...
     P                 E
      *
      *=====================================================================*
      *  Returns cFalse if a given character does not apply to the rules
      *  of a normalized string.
      *=====================================================================*
     P isNormalizedStringChar...
     P                 B
      *
     D isNormalizedStringChar...
     D                 PI              N
     D  i_char                        1A   const
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         if (i_char = x'0D' or   // carriage return
             i_char = x'25' or   // line feed
             i_char = x'05');    // tab
            return cFalse;
         endif;

         return cTrue;

      /END-FREE
      *
     P isNormalizedStringChar...
     P                 E
      *
      *=====================================================================*
      *  Clears the name and xpath of the field that has been unmarshalled. *
      *=====================================================================*
     P clearFieldIdentifier...
     P                 B
      *
     D clearFieldIdentifier...
     D                 PI
      *
      *  Return value
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         reset g_fieldIdentifier;

         return;

      /END-FREE
      *
     P clearFieldIdentifier...
     P                 E
      *
      *=====================================================================*
      *  Produces a message to signal an unexpected unmarshaller error.
      *=====================================================================*
     P produceUnexpectedErrorMessage...
     P                 B
      *
     D produceUnexpectedErrorMessage...
     D                 PI                         likeds(msg_t          )
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D msg             DS                         likeds(msg_t ) inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         msg = produceUnexpectedErrorMessageIntern(i_value);

         return msg;

      /END-FREE
      *
     P produceUnexpectedErrorMessage...
     P                 E
      *
      *=====================================================================*
      *  Produces a message to signal an unexpected unmarshaller error.
      *=====================================================================*
     P produceUnexpectedErrorMessageV6...
     P                 B
      *
     D produceUnexpectedErrorMessageV6...
     D                 PI                         likeds(msg_t          )
     D  i_pValue                       *   const
     D  i_length                     10I 0 value
      *
      *  Return value
     D msg             DS                         likeds(msg_t ) inz
      *
      *  Local fields
     D tmpValue        DS
     D  value                       512A   varying
     D  tmpLength              1      2I 0 inz
     D  tmpData                3    514A   inz
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         if (i_length > 0);
            if (i_length > %size(tmpData));
               i_length = %size(tmpData);
            endif;
            memcpy(%addr(tmpData): i_pValue: i_length);
            tmpLength = i_length;
         else;
            value = '*N';
         endif;

         msg = produceUnexpectedErrorMessageIntern(value);

         return msg;

      /END-FREE
      *
     P produceUnexpectedErrorMessageV6...
     P                 E
      *
      *=====================================================================*
      *  Produces a message to signal an unexpected unmarshaller error.
      *=====================================================================*
     P produceUnexpectedErrorMessageIntern...
     P                 B
      *
     D produceUnexpectedErrorMessageIntern...
     D                 PI                         likeds(msg_t          )
     D  i_value                   65535A   const  varying  options(*varsize)
      *
      *  Return value
     D msg             DS                         likeds(msg_t ) inz
      *
      *  Local fields
     D escMsg          DS                  likeds(msg_t ) inz
     D text            S            512A   varying inz('*N')
      *
      *  Messages
      /COPY QWSDL2RPG,USR5023                      Failed unmarshalling value '&2' of field
      /COPY QWSDL2RPG,USR5024                      Failed unmarshalling value '&2' of array
      * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      /FREE

         escMsg = f_rcvPgmMsgObj(
           cMsg_Escape: *omit: *omit: 0: 2: cMsg_ToMyself: *omit: *omit: *omit);

         if (not AppMsg_isNull(escMsg));
            text = f_rtvMsgText(escMsg: *omit);
         else;
            text = '*N';
         endif;

         if (g_fieldIdentifier.index > 0);
            clear USR5024;
            USR50241 = g_fieldIdentifier.name;            // field name
            USR50242 = i_value;                           // field value
            USR50243 = g_fieldIdentifier.path + '/' +     // xpath
                       g_fieldIdentifier.name;
            USR50244 = text;                              // message text
            USR50245 = g_fieldIdentifier.index;           // array index
            msg = AppMsg_newError('USR5024': USR5024);
         else;
            clear USR5023;
            USR50231 = g_fieldIdentifier.name;            // field name
            USR50232 = i_value;                           // field value
            USR50233 = g_fieldIdentifier.path + '/' +     // xpath
                       g_fieldIdentifier.name;
            USR50234 = text;                              // message text
            msg = AppMsg_newError('USR5023': USR5023);
         endif;

         return msg;

      /END-FREE
      *
     P produceUnexpectedErrorMessageIntern...
     P                 E
      *
